@@ -76,20 +76,25 @@ RZ_API void rz_analysis_class_recover_from_rzbin(RzAnalysis *analysis) {
 	rz_cons_break_pop();
 }
 
-RZ_API void rz_analysis_class_create(RzAnalysis *analysis, const char *name) {
+RZ_API RzAnalysisClassErr rz_analysis_class_create(RzAnalysis *analysis, const char *name) {
 	char *name_sanitized = rz_str_sanitize_sdb_key(name);
 	if (!name_sanitized) {
-		return;
+		return RZ_ANALYSIS_CLASS_ERR_OTHER;
 	}
+	RzAnalysisClassErr err = RZ_ANALYSIS_CLASS_ERR_SUCCESS;
 	const char *key = key_class(name_sanitized);
 	if (!sdb_exists(analysis->sdb_classes, key)) {
 		sdb_set(analysis->sdb_classes, key, "c", 0);
+		RzEventClass event = { .name = name_sanitized };
+		if (analysis->ev) {
+			rz_event_send(analysis->ev, RZ_EVENT_CLASS_NEW, &event);
+		}
+	} else {
+		err = RZ_ANALYSIS_CLASS_ERR_CLASH;
 	}
 
-	RzEventClass event = { .name = name_sanitized };
-	rz_event_send(analysis->ev, RZ_EVENT_CLASS_NEW, &event);
-
 	free(name_sanitized);
+	return err;
 }
 
 RZ_API void rz_analysis_class_delete(RzAnalysis *analysis, const char *name) {
@@ -152,7 +157,9 @@ RZ_API void rz_analysis_class_delete(RzAnalysis *analysis, const char *name) {
 	}
 
 	RzEventClass event = { .name = class_name_sanitized };
-	rz_event_send(analysis->ev, RZ_EVENT_CLASS_DEL, &event);
+	if (analysis->ev) {
+		rz_event_send(analysis->ev, RZ_EVENT_CLASS_DEL, &event);
+	}
 
 	free(class_name_sanitized);
 }
@@ -262,7 +269,9 @@ RZ_API RzAnalysisClassErr rz_analysis_class_rename(RzAnalysis *analysis, const c
 		.name_old = old_name_sanitized,
 		.name_new = new_name_sanitized
 	};
-	rz_event_send(analysis->ev, RZ_EVENT_CLASS_RENAME, &event);
+	if (analysis->ev) {
+		rz_event_send(analysis->ev, RZ_EVENT_CLASS_RENAME, &event);
+	}
 
 beach:
 	free(old_name_sanitized);
@@ -335,7 +344,9 @@ static RzAnalysisClassErr rz_analysis_class_set_attr_raw(RzAnalysis *analysis, c
 			.attr_id = attr_id },
 		.content = content
 	};
-	rz_event_send(analysis->ev, RZ_EVENT_CLASS_ATTR_SET, &event);
+	if (analysis->ev) {
+		rz_event_send(analysis->ev, RZ_EVENT_CLASS_ATTR_SET, &event);
+	}
 
 	return RZ_ANALYSIS_CLASS_ERR_SUCCESS;
 }
@@ -389,7 +400,9 @@ static RzAnalysisClassErr rz_analysis_class_delete_attr_raw(RzAnalysis *analysis
 		.attr_type = attr_type,
 		.attr_id = attr_id
 	};
-	rz_event_send(analysis->ev, RZ_EVENT_CLASS_ATTR_DEL, &event);
+	if (analysis->ev) {
+		rz_event_send(analysis->ev, RZ_EVENT_CLASS_ATTR_DEL, &event);
+	}
 
 	return RZ_ANALYSIS_CLASS_ERR_SUCCESS;
 }
@@ -464,7 +477,9 @@ static RzAnalysisClassErr rz_analysis_class_rename_attr_raw(RzAnalysis *analysis
 			.attr_id = attr_id_old },
 		.attr_id_new = attr_id_new
 	};
-	rz_event_send(analysis->ev, RZ_EVENT_CLASS_ATTR_RENAME, &event);
+	if (analysis->ev) {
+		rz_event_send(analysis->ev, RZ_EVENT_CLASS_ATTR_RENAME, &event);
+	}
 
 	return RZ_ANALYSIS_CLASS_ERR_SUCCESS;
 }
@@ -605,14 +620,14 @@ RZ_API void rz_analysis_class_method_fini(RzAnalysisMethod *meth) {
 	free(meth->real_name);
 }
 
-RZ_API void rz_analysis_class_method_recover(RzAnalysis *analysis, RzBinClass *cls, RzList *methods) {
+RZ_API void rz_analysis_class_method_recover(RzAnalysis *analysis, RzBinClass *cls, RzList /*<RzBinSymbol *>*/ *methods) {
 	RzListIter *iter_method;
 	RzBinSymbol *sym;
 	rz_list_sort(methods, &symbol_method_sort_by_addr);
 	rz_list_foreach (methods, iter_method, sym) {
 		if (!rz_analysis_class_method_exists(analysis, cls->name, sym->name)) {
-			//detect constructor or destructor but not implemented
-			//Temporarily set to default
+			// detect constructor or destructor but not implemented
+			// Temporarily set to default
 			RzAnalysisMethod method;
 			method.addr = sym->vaddr;
 			method.vtable_offset = -1;
@@ -742,6 +757,8 @@ RZ_API RzVector /*<RzAnalysisMethod>*/ *rz_analysis_class_method_get_all(RzAnaly
 	}
 	char *key = key_attr_type_attrs(class_name_sanitized, attr_type_id(RZ_ANALYSIS_CLASS_ATTR_TYPE_METHOD));
 	if (!key) {
+		rz_vector_free(vec);
+		free(class_name_sanitized);
 		return NULL;
 	}
 	char *array = sdb_get(analysis->sdb_classes_attrs, key, 0);
@@ -1171,6 +1188,7 @@ RZ_API RzAnalysisClassErr rz_analysis_class_vtable_set(RzAnalysis *analysis, con
 
 	vtable->id = malloc(16);
 	if (!vtable->id) {
+		free(content);
 		return RZ_ANALYSIS_CLASS_ERR_OTHER;
 	}
 	RzAnalysisClassErr err = rz_analysis_class_add_attr_unique(analysis, class_name, RZ_ANALYSIS_CLASS_ATTR_TYPE_VTABLE, content, vtable->id, 16);
@@ -1245,313 +1263,14 @@ RZ_API RzAnalysisClassErr rz_analysis_class_vtable_delete(RzAnalysis *analysis,
 	return err;
 }
 
-// ---- PRINT ----
-
-RZ_API void rz_analysis_class_print(RzAnalysis *analysis, const char *class_name, bool detailed) {
-	rz_cons_printf("[%s", class_name);
-
-	RzVector *bases = rz_analysis_class_base_get_all(analysis, class_name);
-	if (bases) {
-		RzAnalysisBaseClass *base;
-		bool first = true;
-		rz_vector_foreach(bases, base) {
-			if (first) {
-				rz_cons_print(": ");
-				first = false;
-			} else {
-				rz_cons_print(", ");
-			}
-			rz_cons_print(base->class_name);
-		}
-		rz_vector_free(bases);
-	}
-
-	rz_cons_print("]\n");
-
-	if (detailed) {
-		RzVector *vtables = rz_analysis_class_vtable_get_all(analysis, class_name);
-		if (vtables) {
-			RzAnalysisVTable *vtable;
-			rz_vector_foreach(vtables, vtable) {
-				rz_cons_printf("  (vtable at 0x%" PFMT64x, vtable->addr);
-				if (vtable->offset > 0) {
-					rz_cons_printf(" in class at +0x%" PFMT64x ")\n", vtable->offset);
-				} else {
-					rz_cons_print(")\n");
-				}
-			}
-			rz_vector_free(vtables);
-		}
-
-		RzVector *methods = rz_analysis_class_method_get_all(analysis, class_name);
-		if (methods && rz_vector_len(methods) > 0) {
-			RzTable *table = rz_table_new();
-			rz_table_set_columnsf(table, "dsxxs", "nth", "name", "addr", "vt_offset", "type");
-			rz_table_align(table, 2, RZ_TABLE_ALIGN_RIGHT);
-			char *method_type[] = { "DEFAULT", "VIRTUAL", "V_DESTRUCTOR", "DESTRUCTOR", "CONSTRUCTOR" };
-			RzAnalysisMethod *meth;
-			int i = 1;
-			rz_vector_foreach(methods, meth) {
-				RzList *row_list = rz_list_newf(free);
-				rz_list_append(row_list, rz_str_newf("%d", i++));
-				rz_list_append(row_list, rz_str_new(meth->real_name));
-				rz_list_append(row_list, rz_str_newf("0x%" PFMT64x, meth->addr));
-				if (meth->vtable_offset >= 0) {
-					rz_list_append(row_list, rz_str_newf("0x%" PFMT64x, meth->vtable_offset));
-				} else {
-					rz_list_append(row_list, rz_str_new("-1"));
-				}
-				rz_list_append(row_list, rz_str_new(method_type[meth->method_type]));
-				rz_table_add_row_list(table, row_list);
-			}
-			char *s = rz_table_tostring(table);
-			rz_cons_printf("%s\n", s);
-			free(s);
-			rz_table_free(table);
-		}
-		rz_vector_free(methods);
-	}
-}
-
-static void rz_analysis_class_print_cmd(RzAnalysis *analysis, const char *class_name) {
-	RzVector *bases = rz_analysis_class_base_get_all(analysis, class_name);
-	if (bases) {
-		RzAnalysisBaseClass *base;
-		rz_vector_foreach(bases, base) {
-			rz_cons_printf("acb %s %s %" PFMT64u "\n", class_name, base->class_name, base->offset);
-		}
-		rz_vector_free(bases);
-	}
-
-	RzVector *vtables = rz_analysis_class_vtable_get_all(analysis, class_name);
-	if (vtables) {
-		RzAnalysisVTable *vtable;
-		rz_vector_foreach(vtables, vtable) {
-			rz_cons_printf("acv %s 0x%" PFMT64x " %" PFMT64u "\n", class_name, vtable->addr, vtable->offset);
-		}
-		rz_vector_free(vtables);
-	}
-
-	RzVector *methods = rz_analysis_class_method_get_all(analysis, class_name);
-	if (methods) {
-		RzAnalysisMethod *meth;
-		rz_vector_foreach(methods, meth) {
-			rz_cons_printf("acm %s %s 0x%" PFMT64x " %" PFMT64d "\n", class_name, meth->name, meth->addr, meth->vtable_offset);
-		}
-		rz_vector_free(methods);
-	}
-}
-
-RZ_API void rz_analysis_class_json(RzAnalysis *analysis, PJ *j, const char *class_name) {
-	pj_o(j);
-	pj_ks(j, "name", class_name);
-
-	pj_k(j, "bases");
-	pj_a(j);
-	RzVector *bases = rz_analysis_class_base_get_all(analysis, class_name);
-	if (bases) {
-		RzAnalysisBaseClass *base;
-		rz_vector_foreach(bases, base) {
-			pj_o(j);
-			pj_ks(j, "id", base->id);
-			pj_ks(j, "name", base->class_name);
-			pj_kn(j, "offset", base->offset);
-			pj_end(j);
-		}
-		rz_vector_free(bases);
-	}
-	pj_end(j);
-
-	pj_k(j, "vtables");
-	pj_a(j);
-	RzVector *vtables = rz_analysis_class_vtable_get_all(analysis, class_name);
-	if (vtables) {
-		RzAnalysisVTable *vtable;
-		rz_vector_foreach(vtables, vtable) {
-			pj_o(j);
-			pj_ks(j, "id", vtable->id);
-			pj_kn(j, "addr", vtable->addr);
-			pj_kn(j, "offset", vtable->offset);
-			pj_end(j);
-		}
-	}
-	pj_end(j);
-
-	pj_k(j, "methods");
-	pj_a(j);
-	RzVector *methods = rz_analysis_class_method_get_all(analysis, class_name);
-	if (methods) {
-		char *method_type[] = { "DEFAULT", "VIRTUAL", "V_DESTRUCTOR", "DESTRUCTOR", "CONSTRUCTOR" };
-		RzAnalysisMethod *meth;
-		rz_vector_foreach(methods, meth) {
-			pj_o(j);
-			pj_ks(j, "name", meth->real_name);
-			pj_kn(j, "addr", meth->addr);
-			pj_ks(j, "type", method_type[meth->method_type]);
-			if (meth->vtable_offset >= 0) {
-				pj_kn(j, "vtable_offset", (ut64)meth->vtable_offset);
-			}
-			pj_end(j);
-		}
-		rz_vector_free(methods);
-	}
-	pj_end(j);
-
-	pj_end(j);
-}
-
-typedef struct {
-	RzAnalysis *analysis;
-	PJ *j;
-} ListJsonCtx;
-
-static bool rz_analysis_class_list_json_cb(void *user, const char *k, const char *v) {
-	ListJsonCtx *ctx = user;
-	rz_analysis_class_json(ctx->analysis, ctx->j, k);
-	return true;
-}
-
-static void rz_analysis_class_list_json(RzAnalysis *analysis) {
-	PJ *j = pj_new();
-	if (!j) {
-		return;
-	}
-	pj_a(j);
-
-	ListJsonCtx ctx;
-	ctx.analysis = analysis;
-	ctx.j = j;
-	rz_analysis_class_foreach(analysis, rz_analysis_class_list_json_cb, &ctx);
-
-	pj_end(j);
-	rz_cons_printf("%s\n", pj_string(j));
-	pj_free(j);
-}
-
-RZ_API void rz_analysis_class_list(RzAnalysis *analysis, int mode) {
-	if (mode == 'j') {
-		rz_analysis_class_list_json(analysis);
-		return;
-	}
-
-	SdbList *classes = rz_analysis_class_get_all(analysis, mode != '*');
-	SdbListIter *iter;
-	SdbKv *kv;
-	if (mode == '*') {
-		ls_foreach (classes, iter, kv) {
-			// need to create all classes first, so they can be referenced
-			rz_cons_printf("ac %s\n", sdbkv_key(kv));
-		}
-		ls_foreach (classes, iter, kv) {
-			rz_analysis_class_print_cmd(analysis, sdbkv_key(kv));
-		}
-	} else {
-		ls_foreach (classes, iter, kv) {
-			rz_analysis_class_print(analysis, sdbkv_key(kv), mode == 'l');
-		}
-	}
-	ls_free(classes);
-}
-
-RZ_API void rz_analysis_class_list_bases(RzAnalysis *analysis, const char *class_name) {
-	char *class_name_sanitized = rz_str_sanitize_sdb_key(class_name);
-	if (!class_name_sanitized) {
-		return;
-	}
-	if (!rz_analysis_class_exists_raw(analysis, class_name_sanitized)) {
-		free(class_name_sanitized);
-		return;
-	}
-	rz_cons_printf("%s:\n", class_name_sanitized);
-	free(class_name_sanitized);
-
-	RzVector *bases = rz_analysis_class_base_get_all(analysis, class_name);
-	RzAnalysisBaseClass *base;
-	rz_vector_foreach(bases, base) {
-		rz_cons_printf("  %4s %s @ +0x%" PFMT64x "\n", base->id, base->class_name, base->offset);
-	}
-	rz_vector_free(bases);
-}
-
-RZ_API void rz_analysis_class_list_vtables(RzAnalysis *analysis, const char *class_name) {
-	char *class_name_sanitized = rz_str_sanitize_sdb_key(class_name);
-	if (!class_name_sanitized) {
-		return;
-	}
-	if (!rz_analysis_class_exists_raw(analysis, class_name_sanitized)) {
-		free(class_name_sanitized);
-		return;
-	}
-	rz_cons_printf("%s:\n", class_name_sanitized);
-	free(class_name_sanitized);
-
-	RzVector *vtables = rz_analysis_class_vtable_get_all(analysis, class_name);
-	if (vtables) {
-		RzAnalysisVTable *vtable;
-		rz_vector_foreach(vtables, vtable) {
-			rz_cons_printf("  %4s vtable 0x%" PFMT64x " @ +0x%" PFMT64x " size:+0x%" PFMT64x "\n", vtable->id, vtable->addr, vtable->offset, vtable->size);
-		}
-		rz_vector_free(vtables);
-	}
-}
-
-static void list_all_functions_at_vtable_offset(RzAnalysis *analysis, const char *class_name, ut64 offset) {
-	RVTableContext vtableContext;
-	rz_analysis_vtable_begin(analysis, &vtableContext);
-	ut8 function_ptr_size = vtableContext.word_size;
-
-	ut64 func_address;
-	RzVector *vtables = rz_analysis_class_vtable_get_all(analysis, class_name);
-	RzAnalysisVTable *vtable;
-
-	if (!vtables) {
-		return;
-	}
-
-	rz_vector_foreach(vtables, vtable) {
-		if (vtable->size < offset + function_ptr_size || offset % function_ptr_size) {
-			continue;
-		}
-
-		if (vtableContext.read_addr(analysis, vtable->addr + offset, &func_address))
-			rz_cons_printf("Function address: 0x%08" PFMT64x ", in %s vtable %s\n", func_address, class_name, vtable->id);
-	}
-	rz_vector_free(vtables);
-}
-
-RZ_API void rz_analysis_class_list_vtable_offset_functions(RzAnalysis *analysis, const char *class_name, ut64 offset) {
-	if (class_name) {
-		char *class_name_sanitized = rz_str_sanitize_sdb_key(class_name);
-		if (!class_name_sanitized) {
-			return;
-		}
-		if (!rz_analysis_class_exists_raw(analysis, class_name_sanitized)) {
-			free(class_name_sanitized);
-			return;
-		}
-		free(class_name_sanitized);
-
-		list_all_functions_at_vtable_offset(analysis, class_name, offset);
-	} else {
-		SdbList *classes = rz_analysis_class_get_all(analysis, true);
-		SdbListIter *iter;
-		SdbKv *kv;
-		ls_foreach (classes, iter, kv) {
-			const char *name = sdbkv_key(kv);
-			list_all_functions_at_vtable_offset(analysis, name, offset);
-		}
-		ls_free(classes);
-	}
-}
-
 /**
- * @brief Creates RzGraph from class inheritance information where 
+ * @brief Creates RzGraph from class inheritance information where
  *        each node has RzGraphNodeInfo as generic data
- * 
- * @param analysis 
+ *
+ * @param analysis
  * @return RzGraph* NULL if failure
  */
-RZ_API RzGraph *rz_analysis_class_get_inheritance_graph(RzAnalysis *analysis) {
+RZ_API RzGraph /*<RzGraphNodeInfo *>*/ *rz_analysis_class_get_inheritance_graph(RzAnalysis *analysis) {
 	rz_return_val_if_fail(analysis, NULL);
 	RzGraph *class_graph = rz_graph_new();
 	if (!class_graph) {
