@@ -1,3 +1,4 @@
+// SPDX-FileCopyrightText: 2022 Florian Märkl <info@florianmaerkl.de>
 // SPDX-FileCopyrightText: 2019-2020 condret <condr3t@protonmail.com>
 // SPDX-FileCopyrightText: 2019-2020 riq <ricardoquesada@gmail.com>
 // SPDX-License-Identifier: LGPL-3.0-only
@@ -16,6 +17,7 @@
 #include <rz_asm.h>
 #include <rz_analysis.h>
 #include "../../asm/arch/snes/snes_op_table.h"
+#include "../../asm/arch/6502/6502_il.inc"
 
 enum {
 	_6502_FLAGS_C = (1 << 0),
@@ -28,7 +30,7 @@ enum {
 	_6502_FLAGS_BNZ = (_6502_FLAGS_B | _6502_FLAGS_Z | _6502_FLAGS_N),
 };
 
-static void _6502_analysis_update_flags(RzAnalysisOp *op, int flags) {
+static void _6502_analysis_esil_update_flags(RzAnalysisOp *op, int flags) {
 	/* FIXME: 9,$b instead of 8,$b to prevent the bug triggered by: A = 0 - 0xff - 1 */
 	if (flags & _6502_FLAGS_B) {
 		rz_strbuf_append(&op->esil, ",9,$b,C,:=");
@@ -45,114 +47,217 @@ static void _6502_analysis_update_flags(RzAnalysisOp *op, int flags) {
 }
 
 /* ORA, AND, EOR, ADC, STA, LDA, CMP and SBC share this pattern */
-static void _6502_analysis_esil_get_addr_pattern1(RzAnalysisOp *op, const ut8 *data, int len, char *addrbuf, int addrsize) {
+static void _6502_analysis_esil_get_addr_pattern1(RzAnalysisOp *op, const ut8 *data, size_t len,
+	RZ_NULLABLE char *esiladdr_out, int esiladdr_size,
+	RZ_NULLABLE _6502ILAddr *il_out) {
 	if (len < 1) {
 		return;
 	}
 	// turn off bits 5, 6 and 7
-	switch (data[0] & 0x1f) { // 0x1f = b00111111
+	ut16 imm = 0;
+	switch (data[0] & 0x1f) { // 0x1f = b00011111
 	case 0x09: // op #$ff
 		op->cycles = 2;
-		snprintf(addrbuf, addrsize, "0x%02x", (len > 1) ? data[1] : 0);
+		imm = len > 1 ? data[1] : 0;
+		if (esiladdr_out) {
+			snprintf(esiladdr_out, esiladdr_size, "0x%02x", (unsigned int)imm);
+		}
+		if (il_out) {
+			_6502_il_immediate(il_out, imm);
+		}
 		break;
 	case 0x05: // op $ff
 		op->cycles = 3;
-		snprintf(addrbuf, addrsize, "0x%02x", (len > 1) ? data[1] : 0);
+		imm = len > 1 ? data[1] : 0;
+		if (esiladdr_out) {
+			snprintf(esiladdr_out, esiladdr_size, "0x%02x", (unsigned int)imm);
+		}
+		if (il_out) {
+			_6502_il_addr_absolute(il_out, imm);
+		}
 		break;
 	case 0x15: // op $ff,x
 		op->cycles = 4;
-		snprintf(addrbuf, addrsize, "x,0x%02x,+", (len > 1) ? data[1] : 0);
+		imm = len > 1 ? data[1] : 0;
+		if (esiladdr_out) {
+			snprintf(esiladdr_out, esiladdr_size, "x,0x%02x,+", (unsigned int)imm);
+		}
+		if (il_out) {
+			_6502_il_addr_zero_page_reg(il_out, imm, "x");
+		}
 		break;
 	case 0x0d: // op $ffff
 		op->cycles = 4;
-		snprintf(addrbuf, addrsize, "0x%04x", (len > 2) ? (data[1] | data[2] << 8) : 0);
+		imm = (len > 2) ? ((ut16)data[1] | (ut16)data[2] << 8) : 0;
+		if (esiladdr_out) {
+			snprintf(esiladdr_out, esiladdr_size, "0x%04x", (unsigned int)imm);
+		}
+		if (il_out) {
+			_6502_il_addr_absolute(il_out, imm);
+		}
 		break;
 	case 0x1d: // op $ffff,x
 		// FIXME: Add 1 if page boundary is crossed.
 		op->cycles = 4;
-		snprintf(addrbuf, addrsize, "x,0x%04x,+", (len > 2) ? data[1] | data[2] << 8 : 0);
+		imm = (len > 2) ? ((ut16)data[1] | (ut16)data[2] << 8) : 0;
+		if (esiladdr_out) {
+			snprintf(esiladdr_out, esiladdr_size, "x,0x%04x,+", (unsigned int)imm);
+		}
+		if (il_out) {
+			_6502_il_addr_reg(il_out, imm, "x");
+		}
 		break;
 	case 0x19: // op $ffff,y
 		// FIXME: Add 1 if page boundary is crossed.
 		op->cycles = 4;
-		snprintf(addrbuf, addrsize, "y,0x%04x,+", (len > 2) ? data[1] | data[2] << 8 : 0);
+		imm = (len > 2) ? ((ut16)data[1] | (ut16)data[2] << 8) : 0;
+		if (esiladdr_out) {
+			snprintf(esiladdr_out, esiladdr_size, "y,0x%04x,+", (unsigned int)imm);
+		}
+		if (il_out) {
+			_6502_il_addr_reg(il_out, imm, "y");
+		}
 		break;
 	case 0x01: // op ($ff,x)
 		op->cycles = 6;
-		snprintf(addrbuf, addrsize, "x,0x%02x,+,[2]", (len > 1) ? data[1] : 0);
+		imm = (len > 1) ? data[1] : 0;
+		if (esiladdr_out) {
+			snprintf(esiladdr_out, esiladdr_size, "x,0x%02x,+,[2]", (unsigned int)imm);
+		}
+		if (il_out) {
+			_6502_il_addr_indirect_x(il_out, imm);
+		}
 		break;
 	case 0x11: // op ($ff),y
 		// FIXME: Add 1 if page boundary is crossed.
 		op->cycles = 5;
-		snprintf(addrbuf, addrsize, "y,0x%02x,[2],+", (len > 1) ? data[1] : 0);
+		imm = len > 1 ? data[1] : 0;
+		if (esiladdr_out) {
+			snprintf(esiladdr_out, esiladdr_size, "y,0x%02x,[2],+", (unsigned int)imm);
+		}
+		if (il_out) {
+			_6502_il_addr_indirect_y(il_out, imm);
+		}
 		break;
 	}
 }
 
 /* ASL, ROL, LSR, ROR, STX, LDX, DEC and INC share this pattern */
-static void _6502_analysis_esil_get_addr_pattern2(RzAnalysisOp *op, const ut8 *data, int len, char *addrbuf, int addrsize, char reg) {
+static void _6502_analysis_esil_get_addr_pattern2(RzAnalysisOp *op, const ut8 *data, size_t len,
+	char *addrbuf, int addrsize, const char *reg,
+	RZ_NULLABLE _6502ILAddr *il_out) {
 	// turn off bits 5, 6 and 7
 	if (len < 1) {
 		return;
 	}
+	ut16 imm = 0;
 	switch (data[0] & 0x1f) { // 0x1f = b00111111
 	case 0x02: // op #$ff
 		op->cycles = 2;
-		snprintf(addrbuf, addrsize, "0x%02x", (len > 1) ? data[1] : 0);
+		imm = (len > 1) ? data[1] : 0;
+		snprintf(addrbuf, addrsize, "0x%02x", (unsigned int)imm);
+		if (il_out) {
+			_6502_il_immediate(il_out, imm);
+		}
 		break;
-	case 0x0a: //op a
+	case 0x0a: // op a
 		op->cycles = 2;
 		snprintf(addrbuf, addrsize, "a");
+		if (il_out) {
+			_6502_il_accumulator(il_out);
+		}
 		break;
 	case 0x06: // op $ff
 		op->cycles = 5;
-		snprintf(addrbuf, addrsize, "0x%02x", (len > 1) ? data[1] : 0);
+		imm = (len > 1) ? data[1] : 0;
+		snprintf(addrbuf, addrsize, "0x%02x", (unsigned int)imm);
+		if (il_out) {
+			_6502_il_addr_absolute(il_out, imm);
+		}
 		break;
-	case 0x16: // op $ff,x
+	case 0x16: // op $ff,x (or op $ff,y)
 		op->cycles = 6;
-		snprintf(addrbuf, addrsize, "%c,0x%02x,+", reg, (len > 1) ? data[1] : 0);
+		imm = (len > 1) ? data[1] : 0;
+		snprintf(addrbuf, addrsize, "%s,0x%02x,+", reg, imm);
+		if (il_out) {
+			_6502_il_addr_zero_page_reg(il_out, imm, reg);
+		}
 		break;
 	case 0x0e: // op $ffff
 		op->cycles = 6;
-		snprintf(addrbuf, addrsize, "0x%04x", (len > 2) ? data[1] | data[2] << 8 : 0);
+		imm = (len > 2) ? data[1] | data[2] << 8 : 0;
+		snprintf(addrbuf, addrsize, "0x%04x", imm);
+		if (il_out) {
+			_6502_il_addr_absolute(il_out, imm);
+		}
 		break;
-	case 0x1e: // op $ffff,x
+	case 0x1e: // op $ffff,x (or op $ffff,y)
 		op->cycles = 7;
-		snprintf(addrbuf, addrsize, "%c,0x%04x,+", reg, (len > 2) ? data[1] | data[2] << 8 : 0);
+		imm = (len > 2) ? data[1] | data[2] << 8 : 0;
+		snprintf(addrbuf, addrsize, "%s,0x%04x,+", reg, imm);
+		if (il_out) {
+			_6502_il_addr_reg(il_out, imm, reg);
+		}
 		break;
 	}
 }
 
 /* BIT, JMP, JMP(), STY, LDY, CPY, and CPX share this pattern */
-static void _6502_analysis_esil_get_addr_pattern3(RzAnalysisOp *op, const ut8 *data, int len, char *addrbuf, int addrsize, char reg) {
+static void _6502_analysis_esil_get_addr_pattern3(RzAnalysisOp *op, const ut8 *data, size_t len,
+	char *addrbuf, int addrsize, const char *reg,
+	RZ_NULLABLE _6502ILAddr *il_out) {
 	// turn off bits 5, 6 and 7
 	if (len < 1) {
 		return;
 	}
+	ut16 imm;
 	switch (data[0] & 0x1f) { // 0x1f = b00111111
 	case 0x00: // op #$ff
 		op->cycles = 2;
-		snprintf(addrbuf, addrsize, "0x%02x", (len > 1) ? data[1] : 0);
+		imm = (len > 1) ? data[1] : 0;
+		snprintf(addrbuf, addrsize, "0x%02x", imm);
+		if (il_out) {
+			_6502_il_immediate(il_out, imm);
+		}
 		break;
-	case 0x08: //op a
+	case 0x08: // op a
 		op->cycles = 2;
 		snprintf(addrbuf, addrsize, "a");
+		if (il_out) {
+			_6502_il_accumulator(il_out);
+		}
 		break;
 	case 0x04: // op $ff
 		op->cycles = 5;
-		snprintf(addrbuf, addrsize, "0x%02x", (len > 1) ? data[1] : 0);
+		imm = (len > 1) ? data[1] : 0;
+		snprintf(addrbuf, addrsize, "0x%02x", imm);
+		if (il_out) {
+			_6502_il_addr_absolute(il_out, imm);
+		}
 		break;
 	case 0x14: // op $ff,x
 		op->cycles = 6;
-		snprintf(addrbuf, addrsize, "%c,0x%02x,+", reg, (len > 1) ? data[1] : 0);
+		imm = (len > 1) ? data[1] : 0;
+		snprintf(addrbuf, addrsize, "%s,0x%02x,+", reg, imm);
+		if (il_out) {
+			_6502_il_addr_zero_page_reg(il_out, imm, reg);
+		}
 		break;
 	case 0x0c: // op $ffff
 		op->cycles = 6;
-		snprintf(addrbuf, addrsize, "0x%04x", (len > 2) ? data[1] | data[2] << 8 : 0);
+		imm = (len > 2) ? data[1] | data[2] << 8 : 0;
+		snprintf(addrbuf, addrsize, "0x%04x", imm);
+		if (il_out) {
+			_6502_il_addr_absolute(il_out, imm);
+		}
 		break;
 	case 0x1c: // op $ffff,x
 		op->cycles = 7;
-		snprintf(addrbuf, addrsize, "%c,0x%04x,+", reg, (len > 2) ? data[1] | data[2] << 8 : 0);
+		imm = (len > 2) ? data[1] | data[2] << 8 : 0;
+		snprintf(addrbuf, addrsize, "%s,0x%04x,+", reg, imm);
+		if (il_out) {
+			_6502_il_addr_reg(il_out, imm, reg);
+		}
 		break;
 	}
 }
@@ -207,7 +312,7 @@ static void _6502_analysis_esil_inc_reg(RzAnalysisOp *op, ut8 data0, char *sign)
 		break;
 	}
 	rz_strbuf_setf(&op->esil, "%s,%s%s=", reg, sign, sign);
-	_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+	_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
 }
 
 static void _6502_analysis_esil_mov(RzAnalysisOp *op, ut8 data0) {
@@ -246,7 +351,7 @@ static void _6502_analysis_esil_mov(RzAnalysisOp *op, ut8 data0) {
 
 	// don't update NZ on txs
 	if (data0 != 0x9a) {
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
 	}
 }
 
@@ -266,7 +371,7 @@ static void _6502_analysis_esil_pop(RzAnalysisOp *op, ut8 data0) {
 	rz_strbuf_setf(&op->esil, "sp,++=,sp,0x100,+,[1],%s,=", reg);
 
 	if (data0 == 0x68) {
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
 	}
 }
 
@@ -314,11 +419,13 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		return -1;
 	}
 
-	op->size = snes_op_get_size(1, 1, &snes_op[data[0]]); //snes-arch is similar to nes/6502
+	op->size = snes_op_get_size(1, 1, &snes_op[data[0]]); // snes-arch is similar to nes/6502
 	op->addr = addr;
 	op->type = RZ_ANALYSIS_OP_TYPE_UNK;
 	op->id = data[0];
 	rz_strbuf_init(&op->esil);
+	_6502ILAddr il_addr = { 0 };
+	_6502ILAddr *il_addr_ptr = (mask & RZ_ANALYSIS_OP_MASK_IL) ? &il_addr : NULL;
 	switch (data[0]) {
 	case 0x02:
 	case 0x03:
@@ -441,6 +548,9 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		// PC was already incremented by one at this point. Needs to incremented once more
 		// New PC is Interrupt Vector: $fffe. (FIXME: Confirm this is valid for all 6502)
 		rz_strbuf_set(&op->esil, ",1,I,=,0,D,=,flags,0x10,|,0x100,sp,+,=[1],pc,1,+,0xfe,sp,+,=[2],3,sp,-=,0xfffe,[2],pc,=");
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_brk((ut16)addr);
+		}
 		break;
 
 	// FLAGS
@@ -455,13 +565,19 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		// FIXME: what opcode for this?
 		op->type = RZ_ANALYSIS_OP_TYPE_NOP;
 		_6502_analysis_esil_flags(op, data[0]);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_flag(data[0]);
+		}
 		break;
 	// BIT
 	case 0x24: // bit $ff
 	case 0x2c: // bit $ffff
 		op->type = RZ_ANALYSIS_OP_TYPE_MOV;
-		_6502_analysis_esil_get_addr_pattern3(op, data, len, addrbuf, buffsize, 0);
+		_6502_analysis_esil_get_addr_pattern3(op, data, len, addrbuf, buffsize, NULL, il_addr_ptr);
 		rz_strbuf_setf(&op->esil, "%s,[1],0x80,&,!,!,N,=,%s,[1],0x40,&,!,!,V,=,a,%s,[1],&,0xff,&,!,Z,=", addrbuf, addrbuf, addrbuf);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_bit(il_addr_ptr);
+		}
 		break;
 	// ADC
 	case 0x69: // adc #$ff
@@ -475,15 +591,18 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		// FIXME: update V
 		// FIXME: support BCD mode
 		op->type = RZ_ANALYSIS_OP_TYPE_ADD;
-		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize);
+		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize, il_addr_ptr);
 		if (data[0] == 0x69) { // immediate mode
 			rz_strbuf_setf(&op->esil, "%s,a,+=,7,$c,C,a,+=,7,$c,|,C,:=", addrbuf);
 		} else {
 			rz_strbuf_setf(&op->esil, "%s,[1],a,+=,7,$c,C,a,+=,7,$c,|,C,:=", addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
 		// fix Z
 		rz_strbuf_append(&op->esil, ",a,a,=,$z,Z,:=");
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_adc(il_addr_ptr);
+		}
 		break;
 	// SBC
 	case 0xe9: // sbc #$ff
@@ -497,15 +616,18 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		// FIXME: update V
 		// FIXME: support BCD mode
 		op->type = RZ_ANALYSIS_OP_TYPE_SUB;
-		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize);
+		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize, il_addr_ptr);
 		if (data[0] == 0xe9) { // immediate mode
 			rz_strbuf_setf(&op->esil, "C,!,%s,+,a,-=", addrbuf);
 		} else {
 			rz_strbuf_setf(&op->esil, "C,!,%s,[1],+,a,-=", addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_BNZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_BNZ);
 		// fix Z and revert C
 		rz_strbuf_append(&op->esil, ",a,a,=,$z,Z,:=,C,!=");
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_sbc(il_addr_ptr);
+		}
 		break;
 	// ORA
 	case 0x09: // ora #$ff
@@ -515,16 +637,21 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0x1d: // ora $ffff,x
 	case 0x19: // ora $ffff,y
 	case 0x01: // ora ($ff,x)
-	case 0x11: // ora ($ff),y
+	case 0x11: { // ora ($ff),y
 		op->type = RZ_ANALYSIS_OP_TYPE_OR;
-		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize);
-		if (data[0] == 0x09) { // immediate mode
+		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize, il_addr_ptr);
+		bool is_immediate = data[0] == 0x09;
+		if (is_immediate) { // immediate mode
 			rz_strbuf_setf(&op->esil, "%s,a,|=", addrbuf);
 		} else {
 			rz_strbuf_setf(&op->esil, "%s,[1],a,|=", addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_ora(il_addr_ptr);
+		}
 		break;
+	}
 	// AND
 	case 0x29: // and #$ff
 	case 0x25: // and $ff
@@ -535,13 +662,17 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0x21: // and ($ff,x)
 	case 0x31: // and ($ff),y
 		op->type = RZ_ANALYSIS_OP_TYPE_AND;
-		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize);
-		if (data[0] == 0x29) { // immediate mode
+		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize, il_addr_ptr);
+		bool is_immediate = data[0] == 0x29;
+		if (is_immediate) { // immediate mode
 			rz_strbuf_setf(&op->esil, "%s,a,&=", addrbuf);
 		} else {
 			rz_strbuf_setf(&op->esil, "%s,[1],a,&=", addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_and(il_addr_ptr);
+		}
 		break;
 	// EOR
 	case 0x49: // eor #$ff
@@ -553,13 +684,16 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0x41: // eor ($ff,x)
 	case 0x51: // eor ($ff),y
 		op->type = RZ_ANALYSIS_OP_TYPE_XOR;
-		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize);
+		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize, il_addr_ptr);
 		if (data[0] == 0x49) { // immediate mode
 			rz_strbuf_setf(&op->esil, "%s,a,^=", addrbuf);
 		} else {
 			rz_strbuf_setf(&op->esil, "%s,[1],a,^=", addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_eor(il_addr_ptr);
+		}
 		break;
 	// ASL
 	case 0x0a: // asl a
@@ -568,13 +702,16 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0x0e: // asl $ffff
 	case 0x1e: // asl $ffff,x
 		op->type = RZ_ANALYSIS_OP_TYPE_SHL;
+		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, "x", il_addr_ptr);
 		if (data[0] == 0x0a) {
 			rz_strbuf_set(&op->esil, "1,a,<<=,7,$c,C,:=,a,a,=");
 		} else {
-			_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, 'x');
 			rz_strbuf_setf(&op->esil, "1,%s,[1],<<,%s,=[1],7,$c,C,:=", addrbuf, addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_asl(il_addr_ptr);
+		}
 		break;
 	// LSR
 	case 0x4a: // lsr a
@@ -583,13 +720,16 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0x4e: // lsr $ffff
 	case 0x5e: // lsr $ffff,x
 		op->type = RZ_ANALYSIS_OP_TYPE_SHR;
+		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, "x", il_addr_ptr);
 		if (data[0] == 0x4a) {
 			rz_strbuf_set(&op->esil, "1,a,&,C,=,1,a,>>=");
 		} else {
-			_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, 'x');
 			rz_strbuf_setf(&op->esil, "1,%s,[1],&,C,=,1,%s,[1],>>,%s,=[1]", addrbuf, addrbuf, addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_lsr(il_addr_ptr);
+		}
 		break;
 	// ROL
 	case 0x2a: // rol a
@@ -598,13 +738,16 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0x2e: // rol $ffff
 	case 0x3e: // rol $ffff,x
 		op->type = RZ_ANALYSIS_OP_TYPE_ROL;
+		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, "x", il_addr_ptr);
 		if (data[0] == 0x2a) {
 			rz_strbuf_set(&op->esil, "1,a,<<,C,|,a,=,7,$c,C,:=,a,a,=");
 		} else {
-			_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, 'x');
 			rz_strbuf_setf(&op->esil, "1,%s,[1],<<,C,|,%s,=[1],7,$c,C,:=", addrbuf, addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_rol(il_addr_ptr);
+		}
 		break;
 	// ROR
 	case 0x6a: // ror a
@@ -615,13 +758,16 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		// uses N as temporary to hold C value. but in fact,
 		// it is not temporary since in all ROR ops, N will have the value of C
 		op->type = RZ_ANALYSIS_OP_TYPE_ROR;
+		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, "x", il_addr_ptr);
 		if (data[0] == 0x6a) {
 			rz_strbuf_set(&op->esil, "C,N,=,1,a,&,C,=,1,a,>>,7,N,<<,|,a,=");
 		} else {
-			_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, 'x');
 			rz_strbuf_setf(&op->esil, "C,N,=,1,%s,[1],&,C,=,1,%s,[1],>>,7,N,<<,|,%s,=[1]", addrbuf, addrbuf, addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_ror(il_addr_ptr);
+		}
 		break;
 	// INC
 	case 0xe6: // inc $ff
@@ -629,9 +775,12 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0xee: // inc $ffff
 	case 0xfe: // inc $ffff,x
 		op->type = RZ_ANALYSIS_OP_TYPE_STORE;
-		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, 'x');
+		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, "x", il_addr_ptr);
 		rz_strbuf_setf(&op->esil, "%s,++=[1]", addrbuf);
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_inc(il_addr_ptr, true);
+		}
 		break;
 	// DEC
 	case 0xc6: // dec $ff
@@ -639,9 +788,12 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0xce: // dec $ffff
 	case 0xde: // dec $ffff,x
 		op->type = RZ_ANALYSIS_OP_TYPE_STORE;
-		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, 'x');
+		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, "x", il_addr_ptr);
 		rz_strbuf_setf(&op->esil, "%s,--=[1]", addrbuf);
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_inc(il_addr_ptr, false);
+		}
 		break;
 	// INX, INY
 	case 0xe8: // inx
@@ -649,6 +801,9 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		op->cycles = 2;
 		op->type = RZ_ANALYSIS_OP_TYPE_STORE;
 		_6502_analysis_esil_inc_reg(op, data[0], "+");
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_inc_reg(data[0] == 0xe8 ? "x" : "y", true);
+		}
 		break;
 	// DEX, DEY
 	case 0xca: // dex
@@ -656,6 +811,9 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		op->cycles = 2;
 		op->type = RZ_ANALYSIS_OP_TYPE_STORE;
 		_6502_analysis_esil_inc_reg(op, data[0], "-");
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_inc_reg(data[0] == 0xca ? "x" : "y", false);
+		}
 		break;
 	// CMP
 	case 0xc9: // cmp #$ff
@@ -667,45 +825,54 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0xc1: // cmp ($ff,x)
 	case 0xd1: // cmp ($ff),y
 		op->type = RZ_ANALYSIS_OP_TYPE_CMP;
-		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize);
+		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize, il_addr_ptr);
 		if (data[0] == 0xc9) { // immediate mode
 			rz_strbuf_setf(&op->esil, "%s,a,==", addrbuf);
 		} else {
 			rz_strbuf_setf(&op->esil, "%s,[1],a,==", addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_BNZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_BNZ);
 		// invert C, since C=1 when A-M >= 0
 		rz_strbuf_append(&op->esil, ",C,!,C,=");
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_cmp("a", il_addr_ptr);
+		}
 		break;
 	// CPX
 	case 0xe0: // cpx #$ff
 	case 0xe4: // cpx $ff
 	case 0xec: // cpx $ffff
 		op->type = RZ_ANALYSIS_OP_TYPE_CMP;
-		_6502_analysis_esil_get_addr_pattern3(op, data, len, addrbuf, buffsize, 0);
+		_6502_analysis_esil_get_addr_pattern3(op, data, len, addrbuf, buffsize, NULL, il_addr_ptr);
 		if (data[0] == 0xe0) { // immediate mode
 			rz_strbuf_setf(&op->esil, "%s,x,==", addrbuf);
 		} else {
 			rz_strbuf_setf(&op->esil, "%s,[1],x,==", addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_BNZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_BNZ);
 		// invert C, since C=1 when A-M >= 0
 		rz_strbuf_append(&op->esil, ",C,!,C,=");
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_cmp("x", il_addr_ptr);
+		}
 		break;
 	// CPY
 	case 0xc0: // cpy #$ff
 	case 0xc4: // cpy $ff
 	case 0xcc: // cpy $ffff
 		op->type = RZ_ANALYSIS_OP_TYPE_CMP;
-		_6502_analysis_esil_get_addr_pattern3(op, data, len, addrbuf, buffsize, 0);
+		_6502_analysis_esil_get_addr_pattern3(op, data, len, addrbuf, buffsize, NULL, il_addr_ptr);
 		if (data[0] == 0xc0) { // immediate mode
 			rz_strbuf_setf(&op->esil, "%s,y,==", addrbuf);
 		} else {
 			rz_strbuf_setf(&op->esil, "%s,[1],y,==", addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_BNZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_BNZ);
 		// invert C, since C=1 when A-M >= 0
 		rz_strbuf_append(&op->esil, ",C,!,C,=");
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_cmp("y", il_addr_ptr);
+		}
 		break;
 	// BRANCHES
 	case 0x10: // bpl $ffff
@@ -734,6 +901,9 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		// FIXME: add a type of conditional
 		// op->cond = RZ_TYPE_COND_LE;
 		_6502_analysis_esil_ccall(op, data[0]);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_branch(data[0], op->jump);
+		}
 		break;
 	// JSR
 	case 0x20: // jsr $ffff
@@ -746,6 +916,9 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		// control to the following address
 		// stack is on page one and sp is an 8-bit reg: operations must be done like: sp + 0x100
 		rz_strbuf_setf(&op->esil, "1,pc,-,0xff,sp,+,=[2],0x%04" PFMT64x ",pc,=,2,sp,-=", op->jump);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_jsr(op->jump, addr);
+		}
 		break;
 	// JMP
 	case 0x4c: // jmp $ffff
@@ -753,14 +926,20 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		op->type = RZ_ANALYSIS_OP_TYPE_JMP;
 		op->jump = (len > 2) ? data[1] | data[2] << 8 : 0;
 		rz_strbuf_setf(&op->esil, "0x%04" PFMT64x ",pc,=", op->jump);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_jmp(op->jump, false);
+		}
 		break;
-	case 0x6c: // jmp ($ffff)
+	case 0x6c: { // jmp ($ffff)
 		op->cycles = 5;
 		op->type = RZ_ANALYSIS_OP_TYPE_UJMP;
-		// FIXME: how to read memory?
-		// op->jump = data[1] | data[2] << 8;
-		rz_strbuf_setf(&op->esil, "0x%04x,[2],pc,=", len > 2 ? data[1] | data[2] << 8 : 0);
+		ut16 imm = len > 2 ? data[1] | data[2] << 8 : 0;
+		rz_strbuf_setf(&op->esil, "0x%04x,[2],pc,=", imm);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_jmp(imm, true);
+		}
 		break;
+	}
 	// RTS
 	case 0x60: // rts
 		op->eob = true;
@@ -771,6 +950,9 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		// Operation:  PC from Stack, PC + 1 -> PC
 		// stack is on page one and sp is an 8-bit reg: operations must be done like: sp + 0x100
 		rz_strbuf_set(&op->esil, "0x101,sp,+,[2],pc,=,pc,++=,2,sp,+=");
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_rts();
+		}
 		break;
 	// RTI
 	case 0x40: // rti
@@ -782,11 +964,17 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		// Operation: P from Stack, PC from Stack
 		// stack is on page one and sp is an 8-bit reg: operations must be done like: sp + 0x100
 		rz_strbuf_set(&op->esil, "0x101,sp,+,[1],flags,=,0x102,sp,+,[2],pc,=,3,sp,+=");
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_rti();
+		}
 		break;
 	// NOP
 	case 0xea: // nop
 		op->type = RZ_ANALYSIS_OP_TYPE_NOP;
 		op->cycles = 2;
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = rz_il_op_new_nop();
+		}
 		break;
 	// LDA
 	case 0xa9: // lda #$ff
@@ -798,13 +986,16 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0xa1: // lda ($ff,x)
 	case 0xb1: // lda ($ff),y
 		op->type = RZ_ANALYSIS_OP_TYPE_LOAD;
-		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize);
-		if (data[0] == 0xa9) { // immediate mode
+		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize, il_addr_ptr);
+		if (data[0] == 0xa9) {
 			rz_strbuf_setf(&op->esil, "%s,a,=", addrbuf);
 		} else {
 			rz_strbuf_setf(&op->esil, "%s,[1],a,=", addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_ld("a", il_addr_ptr);
+		}
 		break;
 	// LDX
 	case 0xa2: // ldx #$ff
@@ -813,13 +1004,16 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0xae: // ldx $ffff
 	case 0xbe: // ldx $ffff,y
 		op->type = RZ_ANALYSIS_OP_TYPE_LOAD;
-		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, 'y');
+		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, "y", il_addr_ptr);
 		if (data[0] == 0xa2) { // immediate mode
 			rz_strbuf_setf(&op->esil, "%s,x,=", addrbuf);
 		} else {
 			rz_strbuf_setf(&op->esil, "%s,[1],x,=", addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_ld("x", il_addr_ptr);
+		}
 		break;
 	// LDY
 	case 0xa0: // ldy #$ff
@@ -828,13 +1022,16 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0xac: // ldy $ffff
 	case 0xbc: // ldy $ffff,x
 		op->type = RZ_ANALYSIS_OP_TYPE_LOAD;
-		_6502_analysis_esil_get_addr_pattern3(op, data, len, addrbuf, buffsize, 'x');
+		_6502_analysis_esil_get_addr_pattern3(op, data, len, addrbuf, buffsize, "x", il_addr_ptr);
 		if (data[0] == 0xa0) { // immediate mode
 			rz_strbuf_setf(&op->esil, "%s,y,=", addrbuf);
 		} else {
 			rz_strbuf_setf(&op->esil, "%s,[1],y,=", addrbuf);
 		}
-		_6502_analysis_update_flags(op, _6502_FLAGS_NZ);
+		_6502_analysis_esil_update_flags(op, _6502_FLAGS_NZ);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_ld("y", il_addr_ptr);
+		}
 		break;
 	// STA
 	case 0x85: // sta $ff
@@ -845,24 +1042,33 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 	case 0x81: // sta ($ff,x)
 	case 0x91: // sta ($ff),y
 		op->type = RZ_ANALYSIS_OP_TYPE_STORE;
-		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize);
+		_6502_analysis_esil_get_addr_pattern1(op, data, len, addrbuf, buffsize, il_addr_ptr);
 		rz_strbuf_setf(&op->esil, "a,%s,=[1]", addrbuf);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_st("a", il_addr_ptr);
+		}
 		break;
 	// STX
 	case 0x86: // stx $ff
 	case 0x96: // stx $ff,y
 	case 0x8e: // stx $ffff
 		op->type = RZ_ANALYSIS_OP_TYPE_STORE;
-		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, 'y');
+		_6502_analysis_esil_get_addr_pattern2(op, data, len, addrbuf, buffsize, "y", il_addr_ptr);
 		rz_strbuf_setf(&op->esil, "x,%s,=[1]", addrbuf);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_st("x", il_addr_ptr);
+		}
 		break;
 	// STY
 	case 0x84: // sty $ff
 	case 0x94: // sty $ff,x
 	case 0x8c: // sty $ffff
 		op->type = RZ_ANALYSIS_OP_TYPE_STORE;
-		_6502_analysis_esil_get_addr_pattern3(op, data, len, addrbuf, buffsize, 'x');
+		_6502_analysis_esil_get_addr_pattern3(op, data, len, addrbuf, buffsize, "x", il_addr_ptr);
 		rz_strbuf_setf(&op->esil, "y,%s,=[1]", addrbuf);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_st("y", il_addr_ptr);
+		}
 		break;
 	// PHP/PHA
 	case 0x08: // php
@@ -872,6 +1078,13 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		op->stackop = RZ_ANALYSIS_STACK_INC;
 		op->stackptr = 1;
 		_6502_analysis_esil_push(op, data[0]);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			if (data[0] == 0x08) {
+				op->il_op = _6502_il_op_php();
+			} else {
+				op->il_op = _6502_il_op_pha();
+			}
+		}
 		break;
 	// PLP,PLA
 	case 0x28: // plp
@@ -881,6 +1094,13 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		op->stackop = RZ_ANALYSIS_STACK_INC;
 		op->stackptr = -1;
 		_6502_analysis_esil_pop(op, data[0]);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			if (data[0] == 0x28) {
+				op->il_op = _6502_il_op_plp();
+			} else {
+				op->il_op = _6502_il_op_pla();
+			}
+		}
 		break;
 	// TAX,TYA,...
 	case 0xaa: // tax
@@ -890,6 +1110,12 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		op->type = RZ_ANALYSIS_OP_TYPE_MOV;
 		op->cycles = 2;
 		_6502_analysis_esil_mov(op, data[0]);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_transfer(
+				data[0] == 0xaa ? "x" : (data[0] == 0xa8 ? "y" : "a"),
+				data[0] == 0x8a ? "x" : (data[0] == 0x98 ? "y" : "a"),
+				true);
+		}
 		break;
 	case 0x9a: // txs
 		op->type = RZ_ANALYSIS_OP_TYPE_MOV;
@@ -898,18 +1124,24 @@ static int _6502_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8
 		// FIXME: should I get register X a place it here?
 		// op->stackptr = get_register_x();
 		_6502_analysis_esil_mov(op, data[0]);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_transfer("sp", "x", false);
+		}
 		break;
 	case 0xba: // tsx
 		op->type = RZ_ANALYSIS_OP_TYPE_MOV;
 		op->cycles = 2;
 		op->stackop = RZ_ANALYSIS_STACK_GET;
 		_6502_analysis_esil_mov(op, data[0]);
+		if (mask & RZ_ANALYSIS_OP_MASK_IL) {
+			op->il_op = _6502_il_op_transfer("x", "sp", true);
+		}
 		break;
 	}
 	return op->size;
 }
 
-static bool set_reg_profile(RzAnalysis *analysis) {
+static char *get_reg_profile(RzAnalysis *analysis) {
 	char *p =
 		"=PC	pc\n"
 		"=SP	sp\n"
@@ -931,11 +1163,11 @@ static bool set_reg_profile(RzAnalysis *analysis) {
 		"gpr	N	.1	.31	0\n"
 		"gpr	sp	.8	4	0\n"
 		"gpr	pc	.16	5	0\n";
-	return rz_reg_set_profile_string(analysis->reg, p);
+	return strdup(p);
 }
 
 static int esil_6502_init(RzAnalysisEsil *esil) {
-	if (esil->analysis && esil->analysis->reg) { //initial values
+	if (esil->analysis && esil->analysis->reg) { // initial values
 		rz_reg_set_value(esil->analysis->reg, rz_reg_get(esil->analysis->reg, "pc", -1), 0x0000);
 		rz_reg_set_value(esil->analysis->reg, rz_reg_get(esil->analysis->reg, "sp", -1), 0xff);
 		rz_reg_set_value(esil->analysis->reg, rz_reg_get(esil->analysis->reg, "a", -1), 0x00);
@@ -950,17 +1182,27 @@ static int esil_6502_fini(RzAnalysisEsil *esil) {
 	return true;
 }
 
+static int address_bits(RzAnalysis *analysis, int bits) {
+	return 16;
+}
+
+static RzAnalysisILConfig *il_config(RzAnalysis *analysis) {
+	return rz_analysis_il_config_new(16, false, 16);
+}
+
 RzAnalysisPlugin rz_analysis_plugin_6502 = {
 	.name = "6502",
 	.desc = "6502/NES analysis plugin",
 	.license = "LGPL3",
 	.arch = "6502",
 	.bits = 8,
+	.address_bits = address_bits,
 	.op = &_6502_op,
-	.set_reg_profile = &set_reg_profile,
+	.get_reg_profile = &get_reg_profile,
 	.esil = true,
 	.esil_init = esil_6502_init,
 	.esil_fini = esil_6502_fini,
+	.il_config = il_config
 };
 
 #ifndef RZ_PLUGIN_INCORE
