@@ -366,12 +366,12 @@ static int rsp_op(RzAnalysis *analysis, RzAnalysisOp *op, ut64 addr, const ut8 *
 	case RSP_OP_MFC2:
 		op->type = RZ_ANALYSIS_OP_TYPE_MOV;
 		op->dst = parsed_operands[0].value;
-		//op->src[0] = parsed_operands[1].value;
+		// op->src[0] = parsed_operands[1].value;
 		break;
 	case RSP_OP_MTC2:
 		op->type = RZ_ANALYSIS_OP_TYPE_MOV;
 		op->src[0] = parsed_operands[0].value;
-		//op->dst = parsed_operands[1].value;
+		// op->dst = parsed_operands[1].value;
 		break;
 	case RSP_OP_CFC2:
 		op->type = RZ_ANALYSIS_OP_TYPE_MOV;
@@ -681,8 +681,21 @@ static char *get_reg_profile(RzAnalysis *analysis) {
 	return strdup(p);
 }
 
-static int archinfo(RzAnalysis *analysis, int q) {
-	return 4;
+static int archinfo(RzAnalysis *a, RzAnalysisInfoType query) {
+	switch (query) {
+	case RZ_ANALYSIS_ARCHINFO_MIN_OP_SIZE:
+		/* fall-thru */
+	case RZ_ANALYSIS_ARCHINFO_MAX_OP_SIZE:
+		/* fall-thru */
+	case RZ_ANALYSIS_ARCHINFO_TEXT_ALIGN:
+		/* fall-thru */
+	case RZ_ANALYSIS_ARCHINFO_DATA_ALIGN:
+		return 4;
+	case RZ_ANALYSIS_ARCHINFO_CAN_USE_POINTERS:
+		return true;
+	default:
+		return -1;
+	}
 }
 
 RzAnalysisPlugin rz_analysis_plugin_rsp = {
