@@ -4,9 +4,8 @@
 // SPDX-License-Identifier: LGPL-3.0-only
 
 #include <rz_bin.h>
-#include <rz_msg_digest.h>
+#include <rz_hash.h>
 #include <rz_util/rz_log.h>
-#include <rz_util/rz_str_search.h>
 #include "i/private.h"
 
 static RzBinClass *__getClass(RzBinFile *bf, const char *name) {
@@ -20,306 +19,25 @@ static RzBinSymbol *__getMethod(RzBinFile *bf, const char *klass, const char *me
 	return ht_pp_find(bf->o->methods_ht, name, NULL);
 }
 
-static RzBinString *__stringAt(RzBinFile *bf, RzList *ret, ut64 addr) {
-	if (addr != 0 && addr != UT64_MAX) {
-		return ht_up_find(bf->o->strings_db, addr, NULL);
-	}
-	return NULL;
-}
-
-static void print_string(RzBinFile *bf, RzBinString *string, int raw, PJ *pj) {
-	rz_return_if_fail(bf && string);
-
-	int mode = bf->strmode;
-	ut64 addr, vaddr;
-	RzBin *bin = bf->rbin;
-	if (!bin) {
-		return;
-	}
-	const char *section_name, *type_string;
-	RzIO *io = bin->iob.io;
-	if (!io) {
-		return;
-	}
-	RzBinSection *s = rz_bin_get_section_at(bf->o, string->paddr, false);
-	if (s) {
-		string->vaddr = s->vaddr + (string->paddr - s->paddr);
-	}
-	section_name = s ? s->name : "";
-	type_string = rz_bin_string_type(string->type);
-	vaddr = addr = bf->o ? rz_bin_object_get_vaddr(bf->o, string->paddr, string->vaddr) : UT64_MAX;
-
-	// If raw string dump mode, use printf to dump directly to stdout.
-	//  PrintfCallback temp = io->cb_printf;
-	switch (mode) {
-	case RZ_MODE_JSON: {
-		if (pj) {
-			pj_o(pj);
-			pj_kn(pj, "vaddr", vaddr);
-			pj_kn(pj, "paddr", string->paddr);
-			pj_kn(pj, "ordinal", string->ordinal);
-			pj_kn(pj, "size", string->size);
-			pj_kn(pj, "length", string->length);
-			pj_ks(pj, "section", section_name);
-			pj_ks(pj, "type", type_string);
-			pj_ks(pj, "string", string->string);
-			pj_end(pj);
-		}
-	} break;
-	case RZ_MODE_SIMPLEST:
-		io->cb_printf("%s\n", string->string);
-		break;
-	case RZ_MODE_SIMPLE:
-		if (raw == 2) {
-			io->cb_printf("0x%08" PFMT64x " %s\n", addr, string->string);
-		} else {
-			io->cb_printf("%s\n", string->string);
-		}
-		break;
-	case RZ_MODE_RIZINCMD: {
-		char *f_name, *nstr;
-		f_name = strdup(string->string);
-		rz_name_filter(f_name, 512, true);
-		if (bin->prefix) {
-			nstr = rz_str_newf("%s.str.%s", bin->prefix, f_name);
-			io->cb_printf("f %s.str.%s %u @ 0x%08" PFMT64x "\n"
-				      "Cs %u @ 0x%08" PFMT64x "\n",
-				bin->prefix, f_name, string->size, addr,
-				string->size, addr);
-		} else {
-			nstr = rz_str_newf("str.%s", f_name);
-			io->cb_printf("f str.%s %u @ 0x%08" PFMT64x "\n"
-				      "Cs %u @ 0x%08" PFMT64x "\n",
-				f_name, string->size, addr,
-				string->size, addr);
-		}
-		free(nstr);
-		free(f_name);
-		break;
-	}
-	case RZ_MODE_PRINT:
-		io->cb_printf("%03u 0x%08" PFMT64x " 0x%08" PFMT64x " %3u %3u "
-			      "(%s) %5s %s\n",
-			string->ordinal, string->paddr, vaddr,
-			string->length, string->size,
-			section_name, type_string, string->string);
-		break;
-	}
-}
-
-static inline void detected_string_to_bin_string(RzBinString *dst, RzDetectedString *src) {
-	int type = -1;
-	switch (src->type) {
-	case RZ_STRING_ENC_LATIN1:
-		type = RZ_STRING_TYPE_ASCII;
-		break;
-	case RZ_STRING_ENC_UTF8:
-		type = RZ_STRING_TYPE_UTF8;
-		break;
-	case RZ_STRING_ENC_UTF16LE:
-		type = RZ_STRING_TYPE_WIDE_LE;
-		break;
-	case RZ_STRING_ENC_UTF32LE:
-		type = RZ_STRING_TYPE_WIDE32_LE;
-		break;
-	case RZ_STRING_ENC_UTF16BE:
-		type = RZ_STRING_TYPE_WIDE_BE;
-		break;
-	case RZ_STRING_ENC_UTF32BE:
-		type = RZ_STRING_TYPE_WIDE32_BE;
-		break;
-	case RZ_STRING_ENC_GUESS:
-		type = RZ_STRING_TYPE_DETECT;
-		break;
-	default:
-		break;
-	}
-
-	dst->string = src->string;
-	dst->size = src->size;
-	dst->length = src->length;
-	dst->type = type;
-	dst->paddr = src->addr;
-	dst->vaddr = src->addr;
-
-	free(src);
-}
-
-static int string_scan_range(RzList *list, RzBinFile *bf, int min,
-	const ut64 from, const ut64 to, RzStrEnc type, int raw, RzBinSection *section) {
-
-	rz_return_val_if_fail(bf, -1);
-
-	RzListIter *it;
-	RzDetectedString *str;
-	RzBinSection *s = NULL;
-
-	RzList *str_list = rz_list_new();
-	if (!str_list) {
-		return -1;
-	}
-
-	RzUtilStrScanOptions scan_opt = {
-		.buf_size = 2048,
-		.max_uni_blocks = 4,
-		.min_str_length = min,
-		.prefer_big_endian = false
-	};
-
-	int count = rz_scan_strings(bf->buf, str_list, &scan_opt, from, to, type);
-	if (count <= 0) {
-		rz_list_free(str_list);
-		return 0;
-	}
-
-	PJ *pj = NULL;
-	if (bf->strmode == RZ_MODE_JSON && !list) {
-		pj = pj_new();
-		if (pj) {
-			pj_a(pj);
-		}
-	}
-
-	int ord = 0;
-	rz_list_foreach (str_list, it, str) {
-		if (!s) {
-			if (section) {
-				s = section;
-			} else if (bf->o) {
-				s = rz_bin_get_section_at(bf->o, str->addr, false);
-			}
-		}
-
-		RzBinString *bs = RZ_NEW0(RzBinString);
-		detected_string_to_bin_string(bs, str);
-		bs->ordinal = ord++;
-
-		if (s) {
-			bs->vaddr = bs->paddr - s->paddr + s->vaddr;
-		}
-
-		if (list) {
-			if (bf->o) {
-				ht_up_insert(bf->o->strings_db, bs->vaddr, bs);
-			}
-			rz_list_append(list, bs);
-		} else {
-			print_string(bf, bs, raw, pj);
-			rz_bin_string_free(bs);
-		}
-
-		if (from == 0 && to == bf->size) {
-			/* force lookup section at the next one */
-			s = NULL;
-		}
-	}
-
-	if (pj) {
-		pj_end(pj);
-		if (bf->rbin) {
-			RzIO *io = bf->rbin->iob.io;
-			if (io) {
-				io->cb_printf("%s", pj_string(pj));
-			}
-		}
-		pj_free(pj);
-	}
-
-	rz_list_free(str_list);
-	return count;
-}
-
-static bool __isDataSection(RzBinFile *a, RzBinSection *s) {
-	if (s->has_strings || s->is_data) {
-		return true;
-	}
-	if (!s->name) {
-		return false;
-	}
-	// Rust
-	return strstr(s->name, "_const") != NULL;
-}
-
-static void get_strings_range(RzBinFile *bf, RzList *list, int min, int raw, ut64 from, ut64 to, RzBinSection *section) {
-	rz_return_if_fail(bf && bf->buf);
-
-	RzBinPlugin *plugin = rz_bin_file_cur_plugin(bf);
-
-	if (!raw && (!plugin || !plugin->info)) {
-		return;
-	}
-	if (!min) {
-		min = plugin ? plugin->minstrlen : 4;
-	}
-	/* Some plugins return zero, fix it up */
-	if (!min) {
-		min = 4;
-	}
-	if (min < 0) {
-		return;
-	}
-	if (!bf->rbin->is_debugger) {
-		if (!to || to > rz_buf_size(bf->buf)) {
-			to = rz_buf_size(bf->buf);
-		}
-		if (!to) {
-			return;
-		}
-	}
-	if (raw != 2) {
-		ut64 size = to - from;
-		// in case of dump ignore here
-		if (bf->rbin->maxstrbuf && size && size > bf->rbin->maxstrbuf) {
-			if (bf->rbin->verbose) {
-				eprintf("WARNING: bin_strings buffer is too big (0x%08" PFMT64x "). Use -zzz or set bin.maxstrbuf (RZ_BIN_MAXSTRBUF) in rizin (rz_bin)\n",
-					size);
-			}
-			return;
-		}
-	}
-
-	RzStrEnc type;
-	const char *enc = bf->rbin->strenc;
-	if (!enc) {
-		type = RZ_STRING_ENC_GUESS;
-	} else if (!strcmp(enc, "latin1")) {
-		type = RZ_STRING_ENC_LATIN1;
-	} else if (!strcmp(enc, "utf8")) {
-		type = RZ_STRING_ENC_UTF8;
-	} else if (!strcmp(enc, "utf16le")) {
-		type = RZ_STRING_ENC_UTF16LE;
-	} else if (!strcmp(enc, "utf32le")) {
-		type = RZ_STRING_ENC_UTF32LE;
-	} else if (!strcmp(enc, "utf16be")) {
-		type = RZ_STRING_ENC_UTF16BE;
-	} else if (!strcmp(enc, "utf32be")) {
-		type = RZ_STRING_ENC_UTF32BE;
-	} else {
-		eprintf("ERROR: encoding %s not supported\n", enc);
-		return;
-	}
-	string_scan_range(list, bf, min, from, to, type, raw, section);
-}
-
-RZ_IPI RzBinFile *rz_bin_file_new(RzBin *bin, const char *file, ut64 file_sz, int rawstr, int fd, const char *xtrname, Sdb *sdb, bool steal_ptr) {
+RZ_IPI RzBinFile *rz_bin_file_new(RzBin *bin, const char *file, ut64 file_sz, int fd, const char *xtrname, bool steal_ptr) {
 	ut32 bf_id;
 	if (!rz_id_pool_grab_id(bin->ids->pool, &bf_id)) {
 		return NULL;
 	}
 	RzBinFile *bf = RZ_NEW0(RzBinFile);
-	if (bf) {
-		bf->id = bf_id;
-		bf->rbin = bin;
-		bf->file = file ? strdup(file) : NULL;
-		bf->rawstr = rawstr;
-		bf->fd = fd;
-		bf->curxtr = xtrname ? rz_bin_get_xtrplugin_by_name(bin, xtrname) : NULL;
-		bf->sdb = sdb;
-		bf->size = file_sz;
-		bf->xtr_data = rz_list_newf((RzListFree)rz_bin_xtrdata_free);
-		bf->xtr_obj = NULL;
-		bf->sdb = sdb_new0();
+	if (!bf) {
+		return NULL;
 	}
+
+	bf->id = bf_id;
+	bf->rbin = bin;
+	bf->file = RZ_STR_DUP(file);
+	bf->fd = fd;
+	bf->curxtr = xtrname ? rz_bin_get_xtrplugin_by_name(bin, xtrname) : NULL;
+	bf->size = file_sz;
+	bf->xtr_data = rz_list_newf((RzListFree)rz_bin_xtrdata_free);
+	bf->xtr_obj = NULL;
+	bf->sdb = sdb_new0();
 	return bf;
 }
 
@@ -345,6 +63,7 @@ RZ_IPI void rz_bin_file_free(void /*RzBinFile*/ *_bf) {
 	free(bf->file);
 	rz_bin_object_free(bf->o);
 	rz_list_free(bf->xtr_data);
+	sdb_free(bf->sdb);
 	if (bf->id != -1) {
 		// TODO: use rz_storage api
 		rz_id_pool_kick_id(bf->rbin->ids->pool, bf->id);
@@ -365,6 +84,10 @@ static RzBinPlugin *get_plugin_from_buffer(RzBin *bin, const char *pluginname, R
 	if (plugin) {
 		return plugin;
 	}
+	plugin = rz_bin_get_binplugin_by_filename(bin);
+	if (plugin) {
+		return plugin;
+	}
 	return rz_bin_get_binplugin_by_name(bin, "any");
 }
 
@@ -413,24 +136,26 @@ static bool xtr_metadata_match(RzBinXtrData *xtr_data, const char *arch, int bit
 	return bits == iter_bits && !strcmp(iter_arch, arch) && !xtr_data->loaded;
 }
 
-RZ_IPI RzBinFile *rz_bin_file_new_from_buffer(RzBin *bin, const char *file, RzBuffer *buf, int rawstr, RzBinObjectLoadOptions *opts, int fd, const char *pluginname) {
+RZ_IPI RzBinFile *rz_bin_file_new_from_buffer(RzBin *bin, const char *file, RzBuffer *buf, RzBinObjectLoadOptions *opts, int fd, const char *pluginname) {
 	rz_return_val_if_fail(bin && file && buf, NULL);
 
-	RzBinFile *bf = rz_bin_file_new(bin, file, rz_buf_size(buf), rawstr, fd, pluginname, NULL, false);
-	if (bf) {
-		RzListIter *item = rz_list_append(bin->binfiles, bf);
-		bf->buf = rz_buf_ref(buf);
-		RzBinPlugin *plugin = get_plugin_from_buffer(bin, pluginname, bf->buf);
-		RzBinObject *o = rz_bin_object_new(bf, plugin, opts, 0, rz_buf_size(bf->buf));
-		if (!o) {
-			rz_list_delete(bin->binfiles, item);
-			return NULL;
-		}
-		// size is set here because the reported size of the object depends on
-		// if loaded from xtr plugin or partially read
-		if (!o->size) {
-			o->size = rz_buf_size(buf);
-		}
+	RzBinFile *bf = rz_bin_file_new(bin, file, rz_buf_size(buf), fd, pluginname, false);
+	if (!bf) {
+		return NULL;
+	}
+
+	RzListIter *item = rz_list_append(bin->binfiles, bf);
+	bf->buf = rz_buf_ref(buf);
+	RzBinPlugin *plugin = get_plugin_from_buffer(bin, pluginname, bf->buf);
+	RzBinObject *o = rz_bin_object_new(bf, plugin, opts, 0, rz_buf_size(bf->buf));
+	if (!o) {
+		rz_list_delete(bin->binfiles, item);
+		return NULL;
+	}
+	// size is set here because the reported size of the object depends on
+	// if loaded from xtr plugin or partially read
+	if (!o->size) {
+		o->size = rz_buf_size(buf);
 	}
 	return bf;
 }
@@ -460,7 +185,7 @@ RZ_API RzBinFile *rz_bin_file_find_by_arch_bits(RzBin *bin, const char *arch, in
 	return binfile;
 }
 
-RZ_IPI RzBinFile *rz_bin_file_find_by_id(RzBin *bin, ut32 bf_id) {
+RZ_API RzBinFile *rz_bin_file_find_by_id(RzBin *bin, ut32 bf_id) {
 	RzBinFile *bf;
 	RzListIter *iter;
 	rz_list_foreach (bin->binfiles, iter, bf) {
@@ -499,11 +224,10 @@ RZ_API bool rz_bin_file_delete(RzBin *bin, RzBinFile *bf) {
 }
 
 RZ_API RzBinFile *rz_bin_file_find_by_fd(RzBin *bin, ut32 bin_fd) {
+	rz_return_val_if_fail(bin, NULL);
 	RzListIter *iter;
 	RzBinFile *bf;
 
-	rz_return_val_if_fail(bin, NULL);
-
 	rz_list_foreach (bin->binfiles, iter, bf) {
 		if (bf->fd == bin_fd) {
 			return bf;
@@ -555,7 +279,7 @@ RZ_IPI bool rz_bin_file_set_obj(RzBin *bin, RzBinFile *bf, RzBinObject *obj) {
 			return false;
 		}
 		if (!obj->info->lang) {
-			obj->info->lang = rz_bin_lang_tostring(obj->lang);
+			obj->info->lang = rz_bin_language_to_string(obj->lang);
 		}
 	}
 	return true;
@@ -572,12 +296,12 @@ RZ_API bool rz_bin_file_set_cur_by_name(RzBin *bin, const char *name) {
 	return rz_bin_file_set_cur_binfile(bin, bf);
 }
 
-RZ_IPI RzBinFile *rz_bin_file_xtr_load_buffer(RzBin *bin, RzBinXtrPlugin *xtr, const char *filename, RzBuffer *buf, RzBinObjectLoadOptions *obj_opts, int idx, int fd, int rawstr) {
+RZ_IPI RzBinFile *rz_bin_file_xtr_load_buffer(RzBin *bin, RzBinXtrPlugin *xtr, const char *filename, RzBuffer *buf, RzBinObjectLoadOptions *obj_opts, int idx, int fd) {
 	rz_return_val_if_fail(bin && xtr && buf, NULL);
 
 	RzBinFile *bf = rz_bin_file_find_by_name(bin, filename);
 	if (!bf) {
-		bf = rz_bin_file_new(bin, filename, rz_buf_size(buf), rawstr, fd, xtr->name, bin->sdb, false);
+		bf = rz_bin_file_new(bin, filename, rz_buf_size(buf), fd, xtr->name, false);
 		if (!bf) {
 			return NULL;
 		}
@@ -593,14 +317,14 @@ RZ_IPI RzBinFile *rz_bin_file_xtr_load_buffer(RzBin *bin, RzBinXtrPlugin *xtr, c
 	} else if (xtr->extractall_from_bytes) {
 		ut64 sz = 0;
 		const ut8 *bytes = rz_buf_data(buf, &sz);
-		eprintf("TODO: Implement extractall_from_buffer in '%s' xtr.bin plugin\n", xtr->name);
+		RZ_LOG_INFO("TODO: Implement extractall_from_buffer in '%s' xtr.bin plugin\n", xtr->name);
 		bf->xtr_data = xtr->extractall_from_bytes(bin, bytes, sz);
 	}
 	if (bf->xtr_data) {
 		RzListIter *iter;
 		RzBinXtrData *x;
-		//populate xtr_data with baddr and laddr that will be used later on
-		//rz_bin_file_object_new_from_xtr_data
+		// populate xtr_data with baddr and laddr that will be used later on
+		// rz_bin_file_object_new_from_xtr_data
 		rz_list_foreach (bf->xtr_data, iter, x) {
 			x->obj_opts = *obj_opts;
 		}
@@ -625,74 +349,6 @@ RZ_API RzBinPlugin *rz_bin_file_cur_plugin(RzBinFile *bf) {
 	return (bf && bf->o) ? bf->o->plugin : NULL;
 }
 
-// TODO: searchStrings() instead
-RZ_IPI RzList *rz_bin_file_get_strings(RzBinFile *bf, int min, int dump, int raw) {
-	rz_return_val_if_fail(bf, NULL);
-	RzListIter *iter;
-	RzBinSection *section;
-	RzList *ret = dump ? NULL : rz_list_newf(rz_bin_string_free);
-
-	if (!raw && bf && bf->o && bf->o->sections && !rz_list_empty(bf->o->sections)) {
-		RzBinObject *o = bf->o;
-		rz_list_foreach (o->sections, iter, section) {
-			if (__isDataSection(bf, section)) {
-				get_strings_range(bf, ret, min, raw, section->paddr,
-					section->paddr + section->size, section);
-			}
-		}
-		rz_list_foreach (o->sections, iter, section) {
-			if (!section->name) {
-				continue;
-			}
-			/* load objc/swift strings */
-			const int bits = (bf->o && bf->o->info) ? bf->o->info->bits : 32;
-			const int cfstr_size = (bits == 64) ? 32 : 16;
-			const int cfstr_offs = (bits == 64) ? 16 : 8;
-			if (strstr(section->name, "__cfstring")) {
-				int i;
-				// XXX do not walk if bin.strings == 0
-				ut8 *p;
-				if (section->size > bf->size) {
-					continue;
-				}
-				ut8 *sbuf = malloc(section->size);
-				if (!sbuf) {
-					continue;
-				}
-				rz_buf_read_at(bf->buf, section->paddr + cfstr_offs, sbuf, section->size);
-				for (i = 0; i < section->size; i += cfstr_size) {
-					ut8 *buf = sbuf;
-					p = buf + i;
-					if ((i + ((bits == 64) ? 8 : 4)) >= section->size) {
-						break;
-					}
-					ut64 cfstr_vaddr = section->vaddr + i;
-					ut64 cstr_vaddr = (bits == 64) ? rz_read_le64(p) : rz_read_le32(p);
-					RzBinString *s = __stringAt(bf, ret, cstr_vaddr);
-					if (s) {
-						RzBinString *bs = RZ_NEW0(RzBinString);
-						if (bs) {
-							bs->type = s->type;
-							bs->length = s->length;
-							bs->size = s->size;
-							bs->ordinal = s->ordinal;
-							bs->vaddr = cfstr_vaddr;
-							bs->paddr = cfstr_vaddr; // XXX should be paddr instead
-							bs->string = rz_str_newf("cstr.%s", s->string);
-							rz_list_append(ret, bs);
-							ht_up_insert(o->strings_db, bs->vaddr, bs);
-						}
-					}
-				}
-				free(sbuf);
-			}
-		}
-	} else {
-		get_strings_range(bf, ret, min, raw, 0, bf->size, NULL);
-	}
-	return ret;
-}
-
 RZ_API ut64 rz_bin_file_get_baddr(RzBinFile *bf) {
 	if (bf && bf->o) {
 		return bf->o->opts.baseaddr;
@@ -712,26 +368,28 @@ RZ_API bool rz_bin_file_close(RzBin *bin, int bd) {
 	return false;
 }
 
-static inline bool add_file_hash(RzMsgDigest *md, const char *name, RzList *list) {
-	char hash[128];
+static inline bool add_file_hash(RzHashCfg *md, const char *name, RzList /*<RzBinFileHash *>*/ *list) {
+	char hash[256];
 	const ut8 *digest = NULL;
-	RzMsgDigestSize digest_size = 0;
+	RzHashSize digest_size = 0;
 
-	digest = rz_msg_digest_get_result(md, name, &digest_size);
+	digest = rz_hash_cfg_get_result(md, name, &digest_size);
 	if (!digest) {
 		return false;
 	}
 
-	if (!strcmp(name, "entropy")) {
+	if (!strncmp(name, "entropy", strlen("entropy"))) {
 		double entropy = rz_read_be_double(digest);
 		rz_strf(hash, "%f", entropy);
+	} else if (!strcmp(name, "ssdeep")) {
+		rz_strf(hash, "%s", (const char *)digest);
 	} else {
 		rz_hex_bin2str(digest, digest_size, hash);
 	}
 
 	RzBinFileHash *fh = RZ_NEW0(RzBinFileHash);
 	if (!fh) {
-		eprintf("Cannot allocate file hash\n");
+		RZ_LOG_ERROR("Cannot allocate RzBinFileHash\n");
 		return false;
 	}
 
@@ -741,82 +399,61 @@ static inline bool add_file_hash(RzMsgDigest *md, const char *name, RzList *list
 	return true;
 }
 
+static ut64 buf_compute_hashes(const ut8 *buf, ut64 size, void *user) {
+	if (!rz_hash_cfg_update((RzHashCfg *)user, buf, size)) {
+		return 0;
+	}
+	return size;
+}
+
 /**
  * Return a list of RzBinFileHash structures with the hashes md5, sha1, sha256, crc32 and entropy
  * computed over the whole \p bf .
  */
-RZ_API RZ_OWN RzList *rz_bin_file_compute_hashes(RzBin *bin, RzBinFile *bf, ut64 limit) {
+RZ_API RZ_OWN RzList /*<RzBinFileHash *>*/ *rz_bin_file_compute_hashes(RzBin *bin, RzBinFile *bf, ut64 limit) {
 	rz_return_val_if_fail(bin && bf && bf->o, NULL);
-	ut64 buf_len = 0, r = 0;
 	RzBinObject *o = bf->o;
 	RzList *file_hashes = NULL;
-	ut8 *buf = NULL;
-	RzMsgDigest *md = NULL;
-	const size_t blocksize = 64000;
-
-	RzIODesc *iod = rz_io_desc_get(bin->iob.io, bf->fd);
-	if (!iod) {
+	RzHashCfg *md = NULL;
+	RzBuffer *buf = rz_buf_new_with_io_fd(&bin->iob, bf->fd);
+	const ut64 buf_size = rz_buf_size(buf);
+	if (!buf_size) {
+		rz_buf_free(buf);
 		return NULL;
 	}
-
-	buf_len = rz_io_desc_size(iod);
-	if (buf_len > limit) {
+	if (buf_size > limit) {
 		if (bin->verbose) {
-			eprintf("Warning: rz_bin_file_hash: file exceeds bin.hashlimit\n");
+			RZ_LOG_WARN("rz_bin_file_hash: file exceeds bin.hashlimit\n");
 		}
 		return NULL;
 	}
-	buf = malloc(blocksize);
-	if (!buf) {
-		eprintf("Cannot allocate computation buffer\n");
-		return NULL;
-	}
-
 	file_hashes = rz_list_newf((RzListFree)rz_bin_file_hash_free);
 	if (!file_hashes) {
-		eprintf("Cannot allocate list\n");
+		RZ_LOG_ERROR("Cannot allocate file hash list\n");
 		goto rz_bin_file_compute_hashes_bad;
 	}
 
-	md = rz_msg_digest_new();
+	md = rz_hash_cfg_new(bin->hash);
 	if (!md) {
 		goto rz_bin_file_compute_hashes_bad;
 	}
 
-	if (!rz_msg_digest_configure(md, "md5") ||
-		!rz_msg_digest_configure(md, "sha1") ||
-		!rz_msg_digest_configure(md, "sha256") ||
-		!rz_msg_digest_configure(md, "crc32") ||
-		!rz_msg_digest_configure(md, "entropy")) {
+	if (!rz_hash_cfg_configure(md, "md5") ||
+		!rz_hash_cfg_configure(md, "sha1") ||
+		!rz_hash_cfg_configure(md, "sha256") ||
+		!rz_hash_cfg_configure(md, "crc32") ||
+		!rz_hash_cfg_configure(md, "entropy")) {
 		goto rz_bin_file_compute_hashes_bad;
 	}
-	if (!rz_msg_digest_init(md)) {
+	if (!rz_hash_cfg_init(md)) {
 		goto rz_bin_file_compute_hashes_bad;
 	}
 
-	while (r + blocksize < buf_len) {
-		rz_io_desc_seek(iod, r, RZ_IO_SEEK_SET);
-		int b = rz_io_desc_read(iod, buf, blocksize);
-		if (b < 0) {
-			RZ_LOG_ERROR("rz_io_desc_read: can't read\n");
-			goto rz_bin_file_compute_hashes_bad;
-		} else if (!rz_msg_digest_update(md, buf, b)) {
-			goto rz_bin_file_compute_hashes_bad;
-		}
-		r += b;
-	}
-	if (r < buf_len) {
-		rz_io_desc_seek(iod, r, RZ_IO_SEEK_SET);
-		const size_t rem_len = buf_len - r;
-		int b = rz_io_desc_read(iod, buf, rem_len);
-		if (b < 1) {
-			RZ_LOG_ERROR("rz_io_desc_read: can't read\n");
-		} else if (!rz_msg_digest_update(md, buf, b)) {
-			goto rz_bin_file_compute_hashes_bad;
-		}
+	if (rz_buf_fwd_scan(buf, 0, buf_size, buf_compute_hashes, md) != buf_size) {
+		goto rz_bin_file_compute_hashes_bad;
 	}
 
-	if (!rz_msg_digest_final(md)) {
+	if (!rz_hash_cfg_final(md)) {
 		goto rz_bin_file_compute_hashes_bad;
 	}
 
@@ -835,19 +472,22 @@ RZ_API RZ_OWN RzList *rz_bin_file_compute_hashes(RzBin *bin, RzBinFile *bf, ut64
 	}
 
 	// TODO: add here more rows
-	free(buf);
-	rz_msg_digest_free(md);
+	rz_buf_free(buf);
+	rz_hash_cfg_free(md);
 	return file_hashes;
 
 rz_bin_file_compute_hashes_bad:
-	free(buf);
-	rz_msg_digest_free(md);
+	rz_buf_free(buf);
+	rz_hash_cfg_free(md);
 	rz_list_free(file_hashes);
 	return NULL;
 }
 
-// Set new hashes to current RzBinInfo, caller should free the returned RzList
-RZ_API RzList *rz_bin_file_set_hashes(RzBin *bin, RzList /*<RzBinFileHash*/ *new_hashes) {
+/**
+ * \brief Set \p file_hashes on current RzBinInfo
+ * \return RzList of previous file_hashes
+ */
+RZ_API RZ_OWN RzList /*<RzBinFileHash *>*/ *rz_bin_file_set_hashes(RzBin *bin, RZ_OWN RzList /*<RzBinFileHash *>*/ *new_hashes) {
 	rz_return_val_if_fail(bin && bin->cur && bin->cur->o && bin->cur->o->info, NULL);
 	RzBinFile *bf = bin->cur;
 	RzBinInfo *info = bf->o->info;
@@ -907,7 +547,7 @@ RZ_API RzBinSymbol *rz_bin_file_add_method(RzBinFile *bf, const char *klass, con
 
 	RzBinClass *c = rz_bin_file_add_class(bf, klass, NULL, 0);
 	if (!c) {
-		eprintf("Cannot allocate class %s\n", klass);
+		RZ_LOG_ERROR("Cannot allocate RzBinClass for '%s'\n", klass);
 		return NULL;
 	}
 	RzBinSymbol *sym = __getMethod(bf, klass, method);
@@ -923,13 +563,7 @@ RZ_API RzBinSymbol *rz_bin_file_add_method(RzBinFile *bf, const char *klass, con
 	return sym;
 }
 
-RZ_API RzBinField *rz_bin_file_add_field(RzBinFile *binfile, const char *classname, const char *name) {
-	//TODO: add_field into class
-	//eprintf ("TODO add field: %s \n", name);
-	return NULL;
-}
-
-RZ_API RzList *rz_bin_file_get_trycatch(RzBinFile *bf) {
+RZ_API RzList /*<RzBinTrycatch *>*/ *rz_bin_file_get_trycatch(RZ_NONNULL RzBinFile *bf) {
 	rz_return_val_if_fail(bf && bf->o && bf->o->plugin, NULL);
 	if (bf->o->plugin->trycatch) {
 		return bf->o->plugin->trycatch(bf);
@@ -937,7 +571,7 @@ RZ_API RzList *rz_bin_file_get_trycatch(RzBinFile *bf) {
 	return NULL;
 }
 
-RZ_API RzList *rz_bin_file_get_symbols(RzBinFile *bf) {
+RZ_API RzList /*<RzBinSymbol *>*/ *rz_bin_file_get_symbols(RzBinFile *bf) {
 	rz_return_val_if_fail(bf, NULL);
 	RzBinObject *o = bf->o;
 	return o ? (RzList *)rz_bin_object_get_symbols(o) : NULL;
