@@ -7,6 +7,45 @@
 #include <rz_util.h>
 #include "i/private.h"
 
+/**
+ * \brief  Tries to decode the base64 string hold by RzBinString and overwrites it
+ *
+ * \param  bstr  The RzBinString to decode
+ */
+RZ_API void rz_bin_string_decode_base64(RZ_NONNULL RzBinString *bstr) {
+	rz_return_if_fail(bstr);
+
+	char *decoded = bstr->string;
+	do {
+		// ensure to decode base64 strings encoded multiple times.
+		char *tmp = (char *)rz_base64_decode_dyn(decoded, -1);
+		if (!tmp || !rz_str_is_printable(tmp)) {
+			free(tmp);
+			break;
+		}
+		free(decoded);
+		decoded = tmp;
+	} while (1);
+
+	if (decoded == bstr->string) {
+		return;
+	}
+	free(bstr->string);
+	bstr->string = decoded;
+	bstr->length = strlen(decoded);
+	bstr->type = RZ_STRING_ENC_BASE64;
+}
+
+static void bin_object_decode_all_base64_strings(RzList /*<RzBinString *>*/ *strings) {
+	rz_return_if_fail(strings);
+
+	RzBinString *bstr;
+	RzListIter *iter;
+	rz_list_foreach (strings, iter, bstr) {
+		rz_bin_string_decode_base64(bstr);
+	}
+}
+
 RZ_API void rz_bin_mem_free(void *data) {
 	RzBinMem *mem = (RzBinMem *)data;
 	if (mem && mem->mirrors) {
@@ -63,7 +102,7 @@ static int reloc_target_cmp(const void *a, const void *b) {
 
 #undef CMP_CHECK
 
-RZ_API RzBinRelocStorage *rz_bin_reloc_storage_new(RZ_OWN RzList *relocs) {
+RZ_API RzBinRelocStorage *rz_bin_reloc_storage_new(RZ_OWN RzList /*<RzBinReloc *>*/ *relocs) {
 	RzBinRelocStorage *ret = RZ_NEW0(RzBinRelocStorage);
 	if (!ret) {
 		return NULL;
@@ -161,8 +200,7 @@ RZ_IPI void rz_bin_object_free(RzBinObject *o) {
 	rz_list_free(o->libs);
 	rz_bin_reloc_storage_free(o->relocs);
 	rz_list_free(o->sections);
-	rz_list_free(o->strings);
-	ht_up_free(o->strings_db);
+	rz_bin_string_database_free(o->strings);
 	ht_pp_free(o->import_name_symbols);
 	rz_list_free(o->symbols);
 	rz_list_free(o->classes);
@@ -202,7 +240,7 @@ static char *swiftField(const char *dn, const char *cn) {
 	return NULL;
 }
 
-static RzList *classes_from_symbols(RzBinFile *bf) {
+static RzList /*<RzBinClass *>*/ *classes_from_symbols(RzBinFile *bf) {
 	RzBinSymbol *sym;
 	RzListIter *iter;
 	rz_list_foreach (bf->o->symbols, iter, sym) {
@@ -251,7 +289,6 @@ RZ_IPI RzBinObject *rz_bin_object_new(RzBinFile *bf, RzBinPlugin *plugin, RzBinO
 	}
 	o->obj_size = (bytes_sz >= sz + offset) ? sz : 0;
 	o->boffset = offset;
-	o->strings_db = ht_up_new0();
 	o->regstate = NULL;
 	o->classes = rz_list_newf((RzListFree)rz_bin_class_free);
 	o->classes_ht = ht_pp_new0();
@@ -297,7 +334,7 @@ RZ_IPI RzBinObject *rz_bin_object_new(RzBinFile *bf, RzBinPlugin *plugin, RzBinO
 	return o;
 }
 
-static void filter_classes(RzBinFile *bf, RzList *list) {
+static void filter_classes(RzBinFile *bf, RzList /*<RzBinClass *>*/ *list) {
 	HtPU *db = ht_pu_new0();
 	HtPP *ht = ht_pp_new0();
 	RzListIter *iter, *iter2;
@@ -355,7 +392,6 @@ static void rz_bin_object_rebuild_classes_ht(RzBinObject *o) {
 RZ_API int rz_bin_object_set_items(RzBinFile *bf, RzBinObject *o) {
 	rz_return_val_if_fail(bf && o && o->plugin, false);
 
-	int i;
 	RzBin *bin = bf->rbin;
 	RzBinPlugin *p = o->plugin;
 	int minlen = (bf->rbin->minstrlen > 0) ? bf->rbin->minstrlen : p->minstrlen;
@@ -381,7 +417,7 @@ RZ_API int rz_bin_object_set_items(RzBinFile *bf, RzBinObject *o) {
 	}
 	// XXX this is expensive because is O(n^n)
 	if (p->binsym) {
-		for (i = 0; i < RZ_BIN_SPECIAL_SYMBOL_LAST; i++) {
+		for (size_t i = 0; i < RZ_BIN_SPECIAL_SYMBOL_LAST; i++) {
 			o->binsym[i] = p->binsym(bf, i);
 			if (o->binsym[i]) {
 				o->binsym[i]->paddr += o->opts.loadaddr;
@@ -461,18 +497,23 @@ RZ_API int rz_bin_object_set_items(RzBinFile *bf, RzBinObject *o) {
 		}
 	}
 	if (bin->filter_rules & RZ_BIN_REQ_STRINGS) {
+		RzList *strings;
 		if (p->strings) {
-			o->strings = p->strings(bf);
+			strings = p->strings(bf);
 		} else {
 			// when a bin plugin does not provide it's own strings
 			// we always take all the strings found in the binary
 			// the method also converts the paddrs to vaddrs
-			o->strings = rz_bin_file_strings(bf, minlen, true);
+			strings = rz_bin_file_strings(bf, minlen, true);
 		}
+
 		if (bin->debase64) {
-			rz_bin_object_filter_strings(o);
+			bin_object_decode_all_base64_strings(strings);
 		}
-		REBASE_PADDR(o, o->strings, RzBinString);
+		REBASE_PADDR(o, strings, RzBinString);
+
+		// RzBinStrDb becomes the owner of the RzList strings
+		o->strings = rz_bin_string_database_new(strings);
 	}
 
 	if (o->info && RZ_STR_ISEMPTY(o->info->compiler)) {
@@ -613,39 +654,6 @@ RZ_IPI RzBinObject *rz_bin_object_find_by_arch_bits(RzBinFile *bf, const char *a
 	return NULL;
 }
 
-RZ_IPI void rz_bin_object_filter_strings(RzBinObject *bo) {
-	rz_return_if_fail(bo && bo->strings);
-
-	RzList *strings = bo->strings;
-	RzBinString *ptr;
-	RzListIter *iter;
-	rz_list_foreach (strings, iter, ptr) {
-		char *dec = (char *)rz_base64_decode_dyn(ptr->string, -1);
-		if (dec) {
-			char *s = ptr->string;
-			for (;;) {
-				char *dec2 = (char *)rz_base64_decode_dyn(s, -1);
-				if (!dec2) {
-					break;
-				}
-				if (!rz_str_is_printable(dec2)) {
-					free(dec2);
-					break;
-				}
-				free(dec);
-				s = dec = dec2;
-			}
-			if (rz_str_is_printable(dec) && strlen(dec) > 3) {
-				free(ptr->string);
-				ptr->string = dec;
-				ptr->type = RZ_BIN_STRING_ENC_BASE64;
-			} else {
-				free(dec);
-			}
-		}
-	}
-}
-
 /**
  * \brief Put the given address on top of o's base address
  */
@@ -698,7 +706,7 @@ RZ_API const RzBinAddr *rz_bin_object_get_special_symbol(RzBinObject *o, RzBinSp
 /**
  * \brief Get list of \p RzBinAddr representing the entry points of the binary object.
  */
-RZ_API const RzList *rz_bin_object_get_entries(RzBinObject *obj) {
+RZ_API const RzList /*<RzBinAddr *>*/ *rz_bin_object_get_entries(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return obj->entries;
 }
@@ -706,7 +714,7 @@ RZ_API const RzList *rz_bin_object_get_entries(RzBinObject *obj) {
 /**
  * \brief Get list of \p RzBinField representing the fields of the binary object.
  */
-RZ_API const RzList *rz_bin_object_get_fields(RzBinObject *obj) {
+RZ_API const RzList /*<RzBinField *>*/ *rz_bin_object_get_fields(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return obj->fields;
 }
@@ -714,7 +722,7 @@ RZ_API const RzList *rz_bin_object_get_fields(RzBinObject *obj) {
 /**
  * \brief Get list of \p RzBinImport representing the imports of the binary object.
  */
-RZ_API const RzList *rz_bin_object_get_imports(RzBinObject *obj) {
+RZ_API const RzList /*<RzBinImport *>*/ *rz_bin_object_get_imports(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return obj->imports;
 }
@@ -722,15 +730,15 @@ RZ_API const RzList *rz_bin_object_get_imports(RzBinObject *obj) {
 /**
  * \brief Get the \p RzBinInfo of the binary object.
  */
-RZ_API const RzBinInfo *rz_bin_object_get_info(RzBinObject *obj) {
+RZ_API const RzBinInfo *rz_bin_object_get_info(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return obj->info;
 }
 
 /**
- * \brief Get list of \p RzBinLib representing the libraries used by the binary object.
+ * \brief Get list of \p char* representing the libraries used by the binary object.
  */
-RZ_API const RzList *rz_bin_object_get_libs(RzBinObject *obj) {
+RZ_API const RzList /*<char *>*/ *rz_bin_object_get_libs(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return obj->libs;
 }
@@ -738,12 +746,12 @@ RZ_API const RzList *rz_bin_object_get_libs(RzBinObject *obj) {
 /**
  * \brief Get list of \p RzBinSection representing both the sections and the segments of the binary object.
  */
-RZ_API const RzList *rz_bin_object_get_sections_all(RzBinObject *obj) {
+RZ_API const RzList /*<RzBinSection *>*/ *rz_bin_object_get_sections_all(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return obj->sections;
 }
 
-static RzList *get_sections_or_segment(RzBinObject *obj, bool is_segment) {
+static RzList /*<RzBinSection *>*/ *get_sections_or_segment(RzBinObject *obj, bool is_segment) {
 	RzList *res = rz_list_new();
 	if (!res) {
 		return NULL;
@@ -762,7 +770,7 @@ static RzList *get_sections_or_segment(RzBinObject *obj, bool is_segment) {
 /**
  * \brief Get list of \p RzBinSection representing only the sections of the binary object.
  */
-RZ_API RzList *rz_bin_object_get_sections(RzBinObject *obj) {
+RZ_API RZ_OWN RzList /*<RzBinSection *>*/ *rz_bin_object_get_sections(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return get_sections_or_segment(obj, false);
 }
@@ -770,15 +778,23 @@ RZ_API RzList *rz_bin_object_get_sections(RzBinObject *obj) {
 /**
  * \brief Get list of \p RzBinSection representing only the segments of the binary object.
  */
-RZ_API RzList *rz_bin_object_get_segments(RzBinObject *obj) {
+RZ_API RZ_OWN RzList /*<RzBinSection *>*/ *rz_bin_object_get_segments(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return get_sections_or_segment(obj, true);
 }
 
+/**
+ * \brief Get list of \p RzBinMap representing only the maps of the binary object.
+ */
+RZ_API RZ_OWN RzList /*<RzBinMap *>*/ *rz_bin_object_get_maps(RZ_NONNULL RzBinObject *obj) {
+	rz_return_val_if_fail(obj, NULL);
+	return obj->maps;
+}
+
 /**
  * \brief Get list of \p RzBinClass representing the classes (e.g. C++ classes) defined in the binary object.
  */
-RZ_API const RzList *rz_bin_object_get_classes(RzBinObject *obj) {
+RZ_API const RzList /*<RzBinClass *>*/ *rz_bin_object_get_classes(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return obj->classes;
 }
@@ -786,15 +802,18 @@ RZ_API const RzList *rz_bin_object_get_classes(RzBinObject *obj) {
 /**
  * \brief Get list of \p RzBinString representing the strings identified in the binary object.
  */
-RZ_API const RzList *rz_bin_object_get_strings(RzBinObject *obj) {
+RZ_API const RzList /*<RzBinString *>*/ *rz_bin_object_get_strings(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
-	return obj->strings;
+	if (!obj->strings) {
+		return NULL;
+	}
+	return obj->strings->list;
 }
 
 /**
  * \brief Get list of \p RzBinMem representing the memory regions identified in the binary object.
  */
-RZ_API const RzList *rz_bin_object_get_mem(RzBinObject *obj) {
+RZ_API const RzList /*<RzBinMem *>*/ *rz_bin_object_get_mem(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return obj->mem;
 }
@@ -802,7 +821,7 @@ RZ_API const RzList *rz_bin_object_get_mem(RzBinObject *obj) {
 /**
  * \brief Get list of \p RzBinSymbol representing the symbols in the binary object.
  */
-RZ_API const RzList *rz_bin_object_get_symbols(RzBinObject *obj) {
+RZ_API const RzList /*<RzBinSymbol *>*/ *rz_bin_object_get_symbols(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return obj->symbols;
 }
@@ -810,7 +829,7 @@ RZ_API const RzList *rz_bin_object_get_symbols(RzBinObject *obj) {
 /**
  * \brief Get a list of \p RzBinResource representing the resources in the binary object.
  */
-RZ_API const RzList *rz_bin_object_get_resources(RzBinObject *obj) {
+RZ_API const RzList /*<RzBinResource *>*/ *rz_bin_object_get_resources(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 	return obj->resources;
 }
@@ -818,56 +837,49 @@ RZ_API const RzList *rz_bin_object_get_resources(RzBinObject *obj) {
 /**
  * \brief Remove all previously identified strings in the binary object and scan it again for strings.
  */
-RZ_API const RzList *rz_bin_object_reset_strings(RzBin *bin, RzBinFile *bf, RzBinObject *obj) {
-	rz_return_val_if_fail(bin && bf && obj, NULL);
-	if (obj->strings) {
-		rz_list_free(obj->strings);
-		obj->strings = NULL;
-	}
-	ht_up_free(obj->strings_db);
-	obj->strings_db = ht_up_new0();
+RZ_API bool rz_bin_object_reset_strings(RZ_NONNULL RzBin *bin, RZ_NONNULL RzBinFile *bf, RZ_NONNULL RzBinObject *obj) {
+	rz_return_val_if_fail(bin && bf && obj, false);
+	RZ_FREE_CUSTOM(obj->strings, rz_bin_string_database_free);
 
+	RzList *strings = NULL;
 	RzBinPlugin *plugin = obj->plugin;
 	if (plugin && plugin->strings) {
-		obj->strings = plugin->strings(bf);
+		strings = plugin->strings(bf);
 	} else {
 		// when a bin plugin does not provide it's own strings
 		// we always take all the strings found in the binary
 		// the method also converts the paddrs to vaddrs
-		obj->strings = rz_bin_file_strings(bf, bin->minstrlen, true);
+		strings = rz_bin_file_strings(bf, bin->minstrlen, true);
 	}
+
 	if (bin->debase64) {
-		rz_bin_object_filter_strings(obj);
+		bin_object_decode_all_base64_strings(strings);
 	}
-	return obj->strings;
+	REBASE_PADDR(obj, strings, RzBinString);
+
+	// RzBinStrDb becomes the owner of the RzList strings
+	obj->strings = rz_bin_string_database_new(strings);
+	return obj->strings != NULL;
 }
 
 /**
- * \brief Return true if at address \p va in the binary object \p obj there is a string
+ * \brief Return RzBinString if at \p address \p there is an entry in the RzBinObject string database
  */
-RZ_API bool rz_bin_object_is_string(RzBinObject *obj, ut64 va) {
+RZ_API RZ_BORROW RzBinString *rz_bin_object_get_string_at(RZ_NONNULL RzBinObject *obj, ut64 address, bool is_va) {
 	rz_return_val_if_fail(obj, false);
-	RzBinString *string;
-	RzListIter *iter;
-	const RzList *list;
-	if (!(list = rz_bin_object_get_strings(obj))) {
-		return false;
+	if (!obj->strings) {
+		return NULL;
 	}
-	rz_list_foreach (list, iter, string) {
-		if (string->vaddr == va) {
-			return true;
-		}
-		if (string->vaddr > va) {
-			return false;
-		}
+	if (is_va) {
+		return ht_up_find(obj->strings->virt, address, NULL);
 	}
-	return false;
+	return ht_up_find(obj->strings->phys, address, NULL);
 }
 
 /**
  * \brief Return true if the binary object \p obj is big endian.
  */
-RZ_API bool rz_bin_object_is_big_endian(RzBinObject *obj) {
+RZ_API bool rz_bin_object_is_big_endian(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, false);
 	return obj->info ? obj->info->big_endian : false;
 }
@@ -875,7 +887,7 @@ RZ_API bool rz_bin_object_is_big_endian(RzBinObject *obj) {
 /**
  * \brief Return true if the binary object \p obj is detected as statically compiled.
  */
-RZ_API bool rz_bin_object_is_static(RzBinObject *obj) {
+RZ_API bool rz_bin_object_is_static(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, false);
 	if (obj->libs && rz_list_length(obj->libs) > 0) {
 		return RZ_BIN_DBG_STATIC & obj->info->dbg_info;
@@ -894,7 +906,7 @@ static void bin_section_map_fini(void *e, void *user) {
  *
  * \return A RzVector* with RzBinSectionMap structure inside.
  **/
-RZ_API RZ_OWN RzVector *rz_bin_object_sections_mapping_list(RzBinObject *obj) {
+RZ_API RZ_OWN RzVector /*<RzBinSectionMap>*/ *rz_bin_object_sections_mapping_list(RZ_NONNULL RzBinObject *obj) {
 	rz_return_val_if_fail(obj, NULL);
 
 	const RzList *all = rz_bin_object_get_sections_all(obj);
@@ -962,7 +974,7 @@ static ut64 map_p2v(RzBinMap *m, ut64 paddr) {
  * \param paddr Offset in the file
  * \return Converted offset to virtual address or UT64_MAX if the conversion cannot be done
  */
-RZ_API ut64 rz_bin_object_p2v(RzBinObject *obj, ut64 paddr) {
+RZ_API ut64 rz_bin_object_p2v(RZ_NONNULL RzBinObject *obj, ut64 paddr) {
 	rz_return_val_if_fail(obj, UT64_MAX);
 	RzBinMap *m = rz_bin_object_get_map_at(obj, paddr, false);
 	if (!m) {
@@ -979,7 +991,7 @@ RZ_API ut64 rz_bin_object_p2v(RzBinObject *obj, ut64 paddr) {
  * \param paddr Offset in the file
  * \return Vector containing \p ut64 values of all possible virtual addresses
  */
-RZ_API RzVector *rz_bin_object_p2v_all(RzBinObject *obj, ut64 paddr) {
+RZ_API RzVector /*<ut64>*/ *rz_bin_object_p2v_all(RZ_NONNULL RzBinObject *obj, ut64 paddr) {
 	rz_return_val_if_fail(obj, NULL);
 	RzPVector *maps = rz_bin_object_get_maps_at(obj, paddr, false);
 	if (!maps) {
@@ -1007,7 +1019,7 @@ RZ_API RzVector *rz_bin_object_p2v_all(RzBinObject *obj, ut64 paddr) {
  * \param paddr Virtual address
  * \return Converted virtual address to offset in the file or UT64_MAX if the conversion cannot be done
  */
-RZ_API ut64 rz_bin_object_v2p(RzBinObject *obj, ut64 vaddr) {
+RZ_API ut64 rz_bin_object_v2p(RZ_NONNULL RzBinObject *obj, ut64 vaddr) {
 	rz_return_val_if_fail(obj, UT64_MAX);
 	RzBinMap *m = rz_bin_object_get_map_at(obj, vaddr, true);
 	if (!m) {
@@ -1020,3 +1032,105 @@ RZ_API ut64 rz_bin_object_v2p(RzBinObject *obj, ut64 vaddr) {
 	}
 	return m->paddr + delta;
 }
+
+/**
+ * \brief   Allocates and initializes the RzBinStrDb structure with the given list of strings
+ *
+ * \param   list  The list of strings to initialize the database with
+ *
+ * \return  On success returns a valid pointer, otherwise NULL
+ */
+RZ_API RZ_OWN RzBinStrDb *rz_bin_string_database_new(RZ_NULLABLE RZ_OWN RzList /*<RzBinString *>*/ *list) {
+	RzBinStrDb *db = RZ_NEW0(RzBinStrDb);
+	if (!db) {
+		RZ_LOG_ERROR("rz_bin: Cannot allocate RzBinStrDb\n");
+		rz_list_free(list);
+		return NULL;
+	}
+
+	db->list = list ? list : rz_list_newf((RzListFree)rz_bin_string_free);
+	db->phys = ht_up_new0();
+	db->virt = ht_up_new0();
+	if (!db->list || !db->phys || !db->virt) {
+		RZ_LOG_ERROR("rz_bin: Cannot allocate RzBinStrDb internal data structure.\n");
+		goto fail;
+	}
+
+	RzListIter *it;
+	RzBinString *bstr;
+	rz_list_foreach (list, it, bstr) {
+		if (!ht_up_update(db->phys, bstr->paddr, bstr)) {
+			RZ_LOG_ERROR("rz_bin: Cannot insert/update RzBinString in RzBinStrDb (phys)\n");
+			goto fail;
+		}
+		if (!ht_up_update(db->virt, bstr->vaddr, bstr)) {
+			RZ_LOG_ERROR("rz_bin: Cannot insert/update RzBinString in RzBinStrDb (virt)\n");
+			goto fail;
+		}
+	}
+	return db;
+
+fail:
+	rz_bin_string_database_free(db);
+	return NULL;
+}
+
+/**
+ * \brief  Frees a RzBinStrDb structure
+ *
+ * \param  db    The string database to free
+ */
+RZ_API void rz_bin_string_database_free(RZ_NULLABLE RzBinStrDb *db) {
+	if (!db) {
+		return;
+	}
+	rz_list_free(db->list);
+	ht_up_free(db->phys);
+	ht_up_free(db->virt);
+	free(db);
+}
+
+/**
+ * \brief   { function_description }
+ *
+ * \param   db    The database
+ * \param   bstr  The bstr
+ *
+ * \return  { description_of_the_return_value }
+ */
+RZ_API bool rz_bin_string_database_add(RZ_NONNULL RzBinStrDb *db, RZ_NONNULL RzBinString *bstr) {
+	rz_return_val_if_fail(db && bstr, false);
+
+	if (!rz_list_append(db->list, bstr)) {
+		RZ_LOG_ERROR("rz_bin: Cannot add RzBinString in RzBinStrDb (list)\n");
+		return false;
+	}
+
+	if (!ht_up_update(db->phys, bstr->paddr, bstr)) {
+		RZ_LOG_ERROR("rz_bin: Cannot add RzBinString in RzBinStrDb (phys)\n");
+		return false;
+	}
+
+	if (!ht_up_update(db->virt, bstr->vaddr, bstr)) {
+		RZ_LOG_ERROR("rz_bin: Cannot add RzBinString in RzBinStrDb (virt)\n");
+		return false;
+	}
+	return true;
+}
+
+/**
+ * \brief Return true if the given \p address \p has been removed to the RzBinObject string database
+ */
+RZ_API bool rz_bin_string_database_remove(RZ_NONNULL RzBinStrDb *db, ut64 address, bool is_va) {
+	rz_return_val_if_fail(db, false);
+
+	RzBinString *bstr = ht_up_find(is_va ? db->virt : db->phys, address, NULL);
+	if (!bstr) {
+		return false;
+	}
+
+	ht_up_delete(db->virt, bstr->vaddr);
+	ht_up_delete(db->phys, bstr->paddr);
+	rz_list_delete_data(db->list, bstr);
+	return true;
+}
