@@ -94,7 +94,30 @@ static RzBinSymbol *le_get_symbol(rz_bin_le_obj_t *bin, ut64 *offset) {
 	return sym;
 }
 
-RzList *le_get_entries(rz_bin_le_obj_t *bin) {
+static bool read_le_entry_bundle_entry(RzBuffer *buf, ut64 addr, LE_entry_bundle_entry *e, LE_entry_bundle_type type) {
+	ut64 offset = addr;
+	switch (type) {
+	case ENTRY16:
+		return rz_buf_read8_offset(buf, &offset, &e->entry_16.flags) &&
+			rz_buf_read_le16_offset(buf, &offset, &e->entry_16.offset);
+	case CALLGATE:
+		return rz_buf_read8_offset(buf, &offset, &e->callgate.flags) &&
+			rz_buf_read_le16_offset(buf, &offset, &e->callgate.offset) &&
+			rz_buf_read_le16_offset(buf, &offset, &e->callgate.callgate_sel);
+	case ENTRY32:
+		return rz_buf_read8_offset(buf, &offset, &e->entry_32.flags) &&
+			rz_buf_read_le32_offset(buf, &offset, &e->entry_32.offset);
+	case FORWARDER:
+		return rz_buf_read8_offset(buf, &offset, &e->forwarder.flags) &&
+			rz_buf_read_le16_offset(buf, &offset, &e->forwarder.import_ord) &&
+			rz_buf_read_le32_offset(buf, &offset, &e->forwarder.offset);
+	default:
+		memset(e, 0, sizeof(LE_entry_bundle_entry));
+		return false;
+	}
+}
+
+RzList /*<char *>*/ *le_get_entries(rz_bin_le_obj_t *bin) {
 	ut64 offset = (ut64)bin->header->enttab + bin->headerOff;
 	RzList *l = rz_list_newf(free);
 	if (!l) {
@@ -124,9 +147,10 @@ RzList *le_get_entries(rz_bin_le_obj_t *bin) {
 		bool typeinfo = header.type & ENTRY_PARAMETER_TYPING_PRESENT;
 		int i;
 		for (i = 0; i < header.count; i++) {
-			ut64 entry = -1;
-			rz_buf_read_at(bin->buf, offset, (ut8 *)&e, sizeof(e));
-			switch (header.type & ~ENTRY_PARAMETER_TYPING_PRESENT) {
+			LE_entry_bundle_type bundle_type = header.type & ~ENTRY_PARAMETER_TYPING_PRESENT;
+			ut64 entry = UT64_MAX;
+			read_le_entry_bundle_entry(bin->buf, offset, &e, bundle_type);
+			switch (bundle_type) {
 			case ENTRY16:
 				if ((header.objnum - 1) < bin->header->objcnt) {
 					entry = (ut64)e.entry_16.offset + bin->objtbl[header.objnum - 1].reloc_base_addr;
@@ -157,6 +181,8 @@ RzList *le_get_entries(rz_bin_le_obj_t *bin) {
 			case FORWARDER:
 				offset += sizeof(e.forwarder);
 				break;
+			default:
+				break;
 			}
 			if (entry != UT64_MAX) {
 				rz_list_append(l, rz_str_newf("0x%" PFMT64x, entry));
@@ -166,7 +192,7 @@ RzList *le_get_entries(rz_bin_le_obj_t *bin) {
 	return l;
 }
 
-static void le_get_symbols_at(rz_bin_le_obj_t *bin, RzList *syml, RzList *entl, ut64 offset, ut64 end) {
+static void le_get_symbols_at(rz_bin_le_obj_t *bin, RzList /*<RzBinSymbol *>*/ *syml, RzList /*<char *>*/ *entl, ut64 offset, ut64 end) {
 	while (offset < end) {
 		RzBinSymbol *sym = le_get_symbol(bin, &offset);
 		if (!sym) {
@@ -188,7 +214,7 @@ static void le_get_symbols_at(rz_bin_le_obj_t *bin, RzList *syml, RzList *entl,
 	}
 }
 
-RzList *rz_bin_le_get_symbols(rz_bin_le_obj_t *bin) {
+RzList /*<RzBinSymbol *>*/ *rz_bin_le_get_symbols(rz_bin_le_obj_t *bin) {
 	RzList *l = rz_list_newf((RzListFree)rz_bin_symbol_free);
 	RzList *entries = le_get_entries(bin);
 	LE_image_header *h = bin->header;
@@ -202,7 +228,7 @@ RzList *rz_bin_le_get_symbols(rz_bin_le_obj_t *bin) {
 	return l;
 }
 
-RzList *rz_bin_le_get_imports(rz_bin_le_obj_t *bin) {
+RzList /*<RzBinImport *>*/ *rz_bin_le_get_imports(rz_bin_le_obj_t *bin) {
 	RzList *l = rz_list_newf((RzListFree)rz_bin_import_free);
 	if (!l) {
 		return NULL;
@@ -226,7 +252,7 @@ RzList *rz_bin_le_get_imports(rz_bin_le_obj_t *bin) {
 	return l;
 }
 
-RzList *rz_bin_le_get_entrypoints(rz_bin_le_obj_t *bin) {
+RzList /*<RzBinAddr *>*/ *rz_bin_le_get_entrypoints(rz_bin_le_obj_t *bin) {
 	RzList *l = rz_list_newf((RzListFree)free);
 	if (!l) {
 		return NULL;
@@ -242,7 +268,7 @@ RzList *rz_bin_le_get_entrypoints(rz_bin_le_obj_t *bin) {
 	return l;
 }
 
-RzList *rz_bin_le_get_libs(rz_bin_le_obj_t *bin) {
+RzList /*<char *>*/ *rz_bin_le_get_libs(rz_bin_le_obj_t *bin) {
 	RzList *l = rz_list_newf((RzListFree)free);
 	if (!l) {
 		return NULL;
@@ -265,7 +291,7 @@ RzList *rz_bin_le_get_libs(rz_bin_le_obj_t *bin) {
  *	page->size is the total size of iter records that describe the page
  *	TODO: Don't do this
  */
-static void __create_iter_sections(RzList *l, rz_bin_le_obj_t *bin, RzBinSection *sec, LE_object_page_entry *page, ut64 vaddr, int cur_page) {
+static void __create_iter_sections(RzList /*<RzBinSection *>*/ *l, rz_bin_le_obj_t *bin, RzBinSection *sec, LE_object_page_entry *page, ut64 vaddr, int cur_page) {
 	rz_return_if_fail(l && bin && sec && page);
 	LE_image_header *h = bin->header;
 	ut32 offset = (h->itermap + (page->offset << (bin->is_le ? 0 : h->pageshift)));
@@ -335,7 +361,7 @@ static void __create_iter_sections(RzList *l, rz_bin_le_obj_t *bin, RzBinSection
 }
 
 // TODO: Compressed page
-RzList *rz_bin_le_get_sections(rz_bin_le_obj_t *bin) {
+RzList /*<RzBinSection *>*/ *rz_bin_le_get_sections(rz_bin_le_obj_t *bin) {
 	RzList *l = rz_list_newf((RzListFree)rz_bin_section_free);
 	if (!l) {
 		return NULL;
@@ -450,7 +476,7 @@ char *le_get_modname_by_ord(rz_bin_le_obj_t *bin, ut32 ordinal) {
 	return modname;
 }
 
-RzList *rz_bin_le_get_relocs(rz_bin_le_obj_t *bin) {
+RzList /*<RzBinReloc *>*/ *rz_bin_le_get_relocs(rz_bin_le_obj_t *bin) {
 	RzList *l = rz_list_newf((RzListFree)free);
 	if (!l) {
 		return NULL;
@@ -488,13 +514,12 @@ RzList *rz_bin_le_get_relocs(rz_bin_le_obj_t *bin) {
 			break;
 		}
 		LE_fixup_record_header header;
-		int ret = rz_buf_read_at(bin->buf, offset, (ut8 *)&header, sizeof(header));
-		if (ret != sizeof(header)) {
+		if (!(rz_buf_read8_offset(bin->buf, &offset, &header.source) &&
+			    rz_buf_read8_offset(bin->buf, &offset, &header.target))) {
 			RZ_LOG_WARN("Cannot read out of bounds relocation.\n");
 			free(rel);
 			break;
 		}
-		offset += sizeof(header);
 		switch (header.source & F_SOURCE_TYPE_MASK) {
 		case BYTEFIXUP:
 			rel->type = RZ_BIN_RELOC_8;
