@@ -64,15 +64,15 @@ static ut64 baddr(RzBinFile *bf) {
 	return MACH0_(get_baddr)(bin);
 }
 
-static RzList *virtual_files(RzBinFile *bf) {
+static RzList /*<RzBinVirtualFile *>*/ *virtual_files(RzBinFile *bf) {
 	return MACH0_(get_virtual_files)(bf);
 }
 
-static RzList *maps(RzBinFile *bf) {
+static RzList /*<RzBinMap *>*/ *maps(RzBinFile *bf) {
 	return MACH0_(get_maps)(bf);
 }
 
-static RzList *sections(RzBinFile *bf) {
+static RzList /*<RzBinSection *>*/ *sections(RzBinFile *bf) {
 	return MACH0_(get_segments)(bf);
 }
 
@@ -93,7 +93,7 @@ static RzBinAddr *newEntry(ut64 hpaddr, ut64 paddr, int type, int bits) {
 	return ptr;
 }
 
-static void process_constructors(RzBinFile *bf, RzList *ret, int bits) {
+static void process_constructors(RzBinFile *bf, RzList /*<RzBinAddr *>*/ *ret, int bits) {
 	RzList *secs = sections(bf);
 	RzListIter *iter;
 	RzBinSection *sec;
@@ -135,10 +135,9 @@ static void process_constructors(RzBinFile *bf, RzList *ret, int bits) {
 			free(buf);
 		}
 	}
-	rz_list_free(secs);
 }
 
-static RzList *entries(RzBinFile *bf) {
+static RzList /*<RzBinAddr *>*/ *entries(RzBinFile *bf) {
 	rz_return_val_if_fail(bf && bf->o, NULL);
 
 	RzBinAddr *ptr = NULL;
@@ -185,22 +184,13 @@ static void _handle_arm_thumb(struct MACH0_(obj_t) * bin, RzBinSymbol **p) {
 	}
 }
 
-#if FEATURE_SYMLIST
-static RzList *symbols(RzBinFile *bf) {
-	RzBinObject *obj = bf ? bf->o : NULL;
-	return (RzList *)MACH0_(get_symbols_list)(obj->bin_obj);
-}
-#else
-static RzList *symbols(RzBinFile *bf) {
+static RzList /*<RzBinSymbol *>*/ *symbols(RzBinFile *bf) {
 	struct MACH0_(obj_t) * bin;
 	int i;
 	const struct symbol_t *syms = NULL;
 	RzBinSymbol *ptr = NULL;
 	RzBinObject *obj = bf ? bf->o : NULL;
 	RzList *ret = rz_list_newf((RzListFree)rz_bin_symbol_free);
-#if 0
-	const char *lang = "c"; // XXX deprecate this
-#endif
 	int wordsize = 0;
 	if (!ret) {
 		return NULL;
@@ -259,16 +249,6 @@ static RzList *symbols(RzBinFile *bf) {
 		ptr->ordinal = i;
 		bin->dbg_info = strncmp(ptr->name, "radr://", 7) ? 0 : 1;
 		set_u_add(symcache, ptr->vaddr);
-#if 0
-		if (!strncmp (ptr->name, "__Z", 3)) {
-			lang = "c++";
-		}
-		if (!strncmp (ptr->name, "type.", 5)) {
-			lang = "go";
-		} else if (!strcmp (ptr->name, "_rust_oom")) {
-			lang = "rust";
-		}
-#endif
 		rz_list_append(ret, ptr);
 	}
 	// functions from LC_FUNCTION_STARTS
@@ -303,20 +283,12 @@ static RzList *symbols(RzBinFile *bf) {
 			}
 		}
 	}
-#if 0
-// this must be done in bobj.c not here
-	if (bin->has_blocks_ext) {
-		lang = !strcmp (lang, "c++") ? "c++ blocks ext." : "c blocks ext.";
-	}
-	bin->lang = lang;
-#endif
 	if (isStripped) {
 		bin->dbg_info |= RZ_BIN_DBG_STRIPPED;
 	}
 	set_u_free(symcache);
 	return ret;
 }
-#endif // FEATURE_SYMLIST
 
 static RzBinImport *import_from_name(RzBin *rbin, const char *orig_name, HtPP *imports_by_name) {
 	if (imports_by_name) {
@@ -362,7 +334,7 @@ static RzBinImport *import_from_name(RzBin *rbin, const char *orig_name, HtPP *i
 	return ptr;
 }
 
-static RzList *imports(RzBinFile *bf) {
+static RzList /*<RzBinImport *>*/ *imports(RzBinFile *bf) {
 	RzBinObject *obj = bf ? bf->o : NULL;
 	struct MACH0_(obj_t) *bin = bf ? bf->o->bin_obj : NULL;
 	const char *name;
@@ -408,7 +380,7 @@ static RzList *imports(RzBinFile *bf) {
 	return ret;
 }
 
-static RzList *relocs(RzBinFile *bf) {
+static RzList /*<RzBinReloc *>*/ *relocs(RzBinFile *bf) {
 	RzList *ret = NULL;
 	struct MACH0_(obj_t) *bin = NULL;
 	RzBinObject *obj = bf ? bf->o : NULL;
@@ -456,7 +428,7 @@ static RzList *relocs(RzBinFile *bf) {
 	return ret;
 }
 
-static RzList *libs(RzBinFile *bf) {
+static RzList /*<char *>*/ *libs(RzBinFile *bf) {
 	int i;
 	char *ptr = NULL;
 	struct lib_t *libs;
@@ -523,7 +495,7 @@ static RzBinInfo *info(RzBinFile *bf) {
 	return ret;
 }
 
-static RzList *classes(RzBinFile *bf) {
+static RzList /*<RzBinClass *>*/ *classes(RzBinFile *bf) {
 	return MACH0_(parse_classes)(bf, NULL);
 }
 
@@ -849,8 +821,8 @@ static ut64 size(RzBinFile *bf) {
 	return off + len;
 }
 
-static RzList *strings(RzBinFile *bf) {
-	return rz_bin_file_strings(bf, 4, false);
+static RzList /*<RzBinString *>*/ *strings(RzBinFile *bf) {
+	return rz_bin_file_strings(bf, bf->minstrlen, false);
 }
 
 RzBinPlugin rz_bin_plugin_mach0 = {
