@@ -3,8 +3,6 @@
 
 #include <rz_debug.h>
 
-#if DEBUGGER
-
 #if __x86_64__ || __i386__ || __arm__ || __arm64__
 #include <sys/uio.h>
 #include <sys/ptrace.h>
@@ -36,6 +34,8 @@ typedef ut64 elf_offset_t;
 #define round_up(a)        ((((a) + (4) - (1)) / (4)) * (4))
 #define sizeof_round_up(b) round_up(sizeof(b))
 
+#define NO_STR_SZ 128
+
 static map_file_t mapping_file = { 0, 0 };
 static note_info_t note_info[NT_LENGHT_T];
 
@@ -143,13 +143,13 @@ static proc_per_thread_t *get_proc_thread_content(int pid, int tid) {
 		return NULL;
 	}
 	{
-		char no_str[128];
+		char no_str[NO_STR_SZ + 1];
 		long unsigned int no_lui;
 		int no_num;
 		char no_char;
 		ut32 no_ui;
-		sscanf(buff, "%d %s %c %d %d %d %d %d %u %lu %lu %lu %lu"
-			     "%" PFMT64x " %" PFMT64x " %ld %lu",
+		sscanf(buff, "%d %" RZ_STR_DEF(NO_STR_SZ) "s %c %d %d %d %d %d %u %lu %lu %lu %lu"
+							  "%" PFMT64x " %" PFMT64x " %ld %lu",
 			&no_num, no_str, &no_char, &no_num, &no_num, &no_num,
 			&no_num, &no_num, &no_ui, &no_lui, &no_lui, &no_lui,
 			&no_lui, &t->utime, &t->stime, &t->cutime, &t->cstime);
@@ -172,19 +172,19 @@ static proc_per_thread_t *get_proc_thread_content(int pid, int tid) {
 		return NULL;
 	}
 	while (!isdigit((ut8)*temp_p_sigpend++)) {
-		//empty body
+		// empty body
 	}
 	p_sigpend = temp_p_sigpend - 1;
 	while (isdigit((ut8)*temp_p_sigpend++)) {
-		//empty body
+		// empty body
 	}
 	p_sigpend[temp_p_sigpend - p_sigpend - 1] = '\0';
 	while (!isdigit((ut8)*temp_p_sighold++)) {
-		//empty body
+		// empty body
 	}
 	p_sighold = temp_p_sighold - 1;
 	while (isdigit((ut8)*temp_p_sighold++)) {
-		//empty body
+		// empty body
 	}
 	p_sighold[temp_p_sighold - p_sighold - 1] = '\0';
 	t->sigpend = atoi(p_sigpend);
@@ -390,7 +390,7 @@ static bool dump_this_map(char *buff_smaps, linux_map_entry_t *entry, ut8 filter
 			goto fail;
 		}
 		while (*flags_str++ == ' ') {
-			//empty body
+			// empty body
 		}
 		flags_str--;
 		p = strtok(flags_str, " ");
@@ -798,12 +798,12 @@ static proc_per_process_t *get_proc_process_content(RzDebug *dbg) {
 	/* /proc/[pid]/stat */
 	/* we only need few fields which are process-wide */
 	{
-		char no_str[128];
+		char no_str[NO_STR_SZ + 1];
 		long unsigned int no_lui;
 		long int no_li;
 		int no_num;
-		sscanf(buff, "%d %s %c %d %d %d %d %d %u %lu %lu %lu %lu"
-			     "%lu %lu %ld %ld %ld %ld %ld",
+		sscanf(buff, "%d %" RZ_STR_DEF(NO_STR_SZ) "s %c %d %d %d %d %d %u %lu %lu %lu %lu"
+							  "%lu %lu %ld %ld %ld %ld %ld",
 			&p->pid, no_str, &p->s_name, &p->ppid, &p->pgrp, &no_num,
 			&no_num, &p->sid, &p->flag, &no_lui, &no_lui, &no_lui,
 			&no_lui, &no_lui, &no_lui, &no_li, &no_li,
@@ -826,11 +826,11 @@ static proc_per_process_t *get_proc_process_content(RzDebug *dbg) {
 	/* Uid */
 	if (temp_p_uid) {
 		while (!isdigit((ut8)*temp_p_uid++)) {
-			//empty body
+			// empty body
 		}
 		p_uid = temp_p_uid - 1;
 		while (isdigit((ut8)*temp_p_uid++)) {
-			//empty body
+			// empty body
 		}
 		p_uid[temp_p_uid - p_uid - 1] = '\0';
 	} else {
@@ -841,11 +841,11 @@ static proc_per_process_t *get_proc_process_content(RzDebug *dbg) {
 	/* Gid */
 	if (temp_p_gid) {
 		while (!isdigit((ut8)*temp_p_gid++)) {
-			//empty body
+			// empty body
 		}
 		p_gid = temp_p_gid - 1;
 		while (isdigit((ut8)*temp_p_gid++)) {
-			//empty body
+			// empty body
 		}
 		p_gid[temp_p_gid - p_gid - 1] = '\0';
 	} else {
@@ -929,8 +929,7 @@ void *linux_get_xsave_data(RzDebug *dbg, int tid, ut32 size) {
 	}
 	transfer.iov_base = xsave_data;
 	transfer.iov_len = size;
-	if (rz_debug_ptrace(dbg, PTRACE_GETREGSET, tid, (void *)NT_X86_XSTATE, &transfer) < 0) {
-		perror("linux_get_xsave_data");
+	if (rz_debug_ptrace_get_x86_xstate(dbg, tid, &transfer) < 0) {
 		free(xsave_data);
 		return NULL;
 	}
@@ -1339,8 +1338,7 @@ static int get_xsave_size(RzDebug *dbg, int pid) {
 	We could also check this by cpuid instruction https://en.wikipedia.org/wiki/CPUID#EAX.3D1:_Processor_Info_and_Feature_Bits*/
 	local.iov_base = xstate_hdr;
 	local.iov_len = sizeof(xstate_hdr);
-	if (rz_debug_ptrace(dbg, PTRACE_GETREGSET, pid, (void *)NT_X86_XSTATE, &local) < 0) {
-		perror("NT_X86_XSTATE");
+	if (rz_debug_ptrace_get_x86_xstate(dbg, pid, &local) < 0) {
 		return 0;
 	}
 
@@ -1539,5 +1537,3 @@ cleanup:
 	return !error;
 }
 #endif
-
-#endif
