@@ -10,6 +10,10 @@
 #include <rz_lib.h>
 #include <string.h>
 
+#define GDB_NAME_SZ   16
+#define GDB_TYPE_SZ   16
+#define GDB_GROUPS_SZ 128
+
 static void rz_reg_profile_def_free(RzRegProfileDef *def) {
 	if (!def) {
 		return;
@@ -511,7 +515,7 @@ static char *gdb_to_rz_profile(const char *gdb) {
 		return NULL;
 	}
 	char *ptr1, *gptr, *gptr1;
-	char name[16], groups[128], type[16];
+	char name[GDB_NAME_SZ + 1], groups[GDB_GROUPS_SZ + 1], type[GDB_TYPE_SZ + 1];
 	const int all = 1, gpr = 2, save = 4, restore = 8, float_ = 16,
 		  sse = 32, vector = 64, system = 128, mmx = 256;
 	int number, rel, offset, size, type_bits, ret;
@@ -542,8 +546,8 @@ static char *gdb_to_rz_profile(const char *gdb) {
 			rz_strbuf_free(sb);
 			return false;
 		}
-		ret = sscanf(ptr, " %s %d %d %d %d %s %s", name, &number, &rel,
-			&offset, &size, type, groups);
+		ret = sscanf(ptr, " %" RZ_STR_DEF(GDB_NAME_SZ) "s %d %d %d %d %" RZ_STR_DEF(GDB_TYPE_SZ) "s %" RZ_STR_DEF(GDB_GROUPS_SZ) "s",
+			name, &number, &rel, &offset, &size, type, groups);
 		// Groups is optional, others not
 		if (ret < 6) {
 			if (*ptr != '*') {
