@@ -7,13 +7,7 @@
 #include <stdio.h>
 #include <string.h>
 
-static int ASN1_STD_FORMAT = 1;
-
-RZ_API void asn1_setformat(int fmt) {
-	ASN1_STD_FORMAT = fmt;
-}
-
-static ut32 asn1_ber_indefinite(const ut8 *buffer, ut32 length) {
+static ut64 asn1_ber_indefinite(const ut8 *buffer, ut64 length) {
 	if (!buffer || length < 3) {
 		return 0;
 	}
@@ -23,10 +17,10 @@ static ut32 asn1_ber_indefinite(const ut8 *buffer, ut32 length) {
 		if (!next[0] && !next[1]) {
 			break;
 		}
-		if (next[0] == 0x80 && (next[-1] & ASN1_FORM) == FORM_CONSTRUCTED) {
+		if (next[0] == 0x80 && (next[-1] & RZ_ASN1_FORM) == RZ_ASN1_FORM_CONSTRUCTED) {
 			next--;
-			int sz = asn1_ber_indefinite(next, end - next);
-			if (sz < 1) {
+			st64 sz = (st64)asn1_ber_indefinite(next, end - next);
+			if (sz < (st64)1) {
 				break;
 			}
 			next += sz;
@@ -36,109 +30,118 @@ static ut32 asn1_ber_indefinite(const ut8 *buffer, ut32 length) {
 	return (next - buffer) + 2;
 }
 
-static RASN1Object *asn1_parse_header(const ut8 *buffer, ut32 length, const ut8 *start_pointer) {
+static RzASN1Object *asn1_parse_header(const ut8 *buffer, ut64 length, const ut8 *start_pointer) {
 	ut8 head, length8, byte;
-	ut64 length64;
+	ut64 length64, remaining;
 	if (!buffer || length < 2) {
 		return NULL;
 	}
 
-	RASN1Object *object = RZ_NEW0(RASN1Object);
+	RzASN1Object *object = RZ_NEW0(RzASN1Object);
 	if (!object) {
 		return NULL;
 	}
 	head = buffer[0];
-	object->offset = start_pointer ? (buffer - start_pointer) : 0;
-	object->klass = head & ASN1_CLASS;
-	object->form = head & ASN1_FORM;
-	object->tag = head & ASN1_TAG;
+	object->offset = buffer - start_pointer;
+	object->klass = head & RZ_ASN1_CLASS;
+	object->form = head & RZ_ASN1_FORM;
+	object->tag = head & RZ_ASN1_TAG;
 	length8 = buffer[1];
-	if (length8 & ASN1_LENLONG) {
+	remaining = length - 2;
+	if (length8 & RZ_ASN1_LENLONG) {
 		length64 = 0;
-		length8 &= ASN1_LENSHORT;
+		length8 &= RZ_ASN1_LENSHORT;
 		object->sector = buffer + 2;
-		if (length8 && length8 < length - 2) {
-			ut8 i8;
+		if (length8 && length8 < remaining) {
+			remaining -= length8;
 			// can overflow.
-			for (i8 = 0; i8 < length8; i8++) {
+			for (ut8 i8 = 0; i8 < length8; i8++) {
 				byte = buffer[2 + i8];
 				length64 <<= 8;
 				length64 |= byte;
-				if (length64 > length) {
+				if (length64 > remaining) {
+					// Malformed object - overflow from data ptr
 					goto out_error;
 				}
 			}
 			object->sector += length8;
 		} else {
-			length64 = asn1_ber_indefinite(object->sector, length - 2);
+			length64 = asn1_ber_indefinite(object->sector, remaining);
+			if (length64 > remaining) {
+				// Malformed object - overflow from data ptr
+				goto out_error;
+			}
 		}
 		object->length = (ut32)length64;
 	} else {
+		if (length8 > remaining) {
+			// Malformed object - overflow from data ptr
+			goto out_error;
+		}
 		object->length = (ut32)length8;
 		object->sector = buffer + 2;
 	}
-
-	if (object->tag == TAG_BITSTRING && object->sector[0] == 0) {
-		if (object->length > 0) {
-			object->sector++; // real sector starts + 1
-			object->length--;
-		}
-	}
-	if (object->length > length) {
+	if (object->sector >= (buffer + length)) {
 		// Malformed object - overflow from data ptr
 		goto out_error;
 	}
+
+	if (object->tag == RZ_ASN1_TAG_BITSTRING && !object->sector[0] && object->length > 0) {
+		object->sector++; // real sector starts + 1
+		object->length--;
+	}
 	return object;
 out_error:
 	free(object);
 	return NULL;
 }
 
-static ut32 rz_asn1_count_objects(const ut8 *buffer, ut32 length) {
-	if (!buffer || !length) {
+static ut32 asn1_count_objects(RzASN1Object *object) {
+	if (!object) {
 		return 0;
 	}
+	const ut8 *buffer = object->sector;
+	ut64 length = object->length;
 	ut32 counter = 0;
-	RASN1Object *object = NULL;
+	RzASN1Object *tmp = NULL;
 	const ut8 *next = buffer;
 	const ut8 *end = buffer + length;
 	while (next >= buffer && next < end) {
 		// i do not care about the offset now.
-		object = asn1_parse_header(next, end - next, 0);
-		if (!object || next == object->sector) {
-			RZ_FREE(object);
+		tmp = asn1_parse_header(next, end - next, buffer);
+		if (!tmp || next == tmp->sector) {
+			RZ_FREE(tmp);
 			break;
 		}
-		next = object->sector + object->length;
+		next = tmp->sector + tmp->length;
 		counter++;
-		RZ_FREE(object);
+		RZ_FREE(tmp);
 	}
-	RZ_FREE(object);
+	RZ_FREE(tmp);
 	return counter;
 }
 
-RZ_API RASN1Object *rz_asn1_create_object(const ut8 *buffer, ut32 length, const ut8 *start_pointer) {
-	RASN1Object *object = asn1_parse_header(buffer, length, start_pointer);
-	if (object && (object->form == FORM_CONSTRUCTED || object->tag == TAG_BITSTRING || object->tag == TAG_OCTETSTRING)) {
+static RzASN1Object *asn1_create_object(const ut8 *buffer, ut64 length, const ut8 *start_pointer) {
+	RzASN1Object *object = asn1_parse_header(buffer, length, start_pointer);
+	if (object && (object->form == RZ_ASN1_FORM_CONSTRUCTED || object->tag == RZ_ASN1_TAG_BITSTRING || object->tag == RZ_ASN1_TAG_OCTETSTRING)) {
 		const ut8 *next = object->sector;
 		const ut8 *end = next + object->length;
 		if (end > buffer + length) {
 			free(object);
 			return NULL;
 		}
-		ut32 count = rz_asn1_count_objects(object->sector, object->length);
+		ut64 count = asn1_count_objects(object);
 		if (count > 0) {
 			object->list.length = count;
-			object->list.objects = RZ_NEWS0(RASN1Object *, count);
+			object->list.objects = RZ_NEWS0(RzASN1Object *, count);
 			if (!object->list.objects) {
-				rz_asn1_free_object(object);
+				rz_asn1_object_free(object);
 				return NULL;
 			}
-			ut32 i;
-			for (i = 0; next >= buffer && next < end && i < count; i++) {
-				RASN1Object *inner = rz_asn1_create_object(next, end - next, start_pointer);
+			for (ut32 i = 0; next >= buffer && next < end && i < count; i++) {
+				RzASN1Object *inner = asn1_create_object(next, end - next, start_pointer);
 				if (!inner || next == inner->sector) {
-					rz_asn1_free_object(inner);
+					rz_asn1_object_free(inner);
 					break;
 				}
 				next = inner->sector + inner->length;
@@ -149,7 +152,28 @@ RZ_API RASN1Object *rz_asn1_create_object(const ut8 *buffer, ut32 length, const
 	return object;
 }
 
-RZ_API RASN1Binary *rz_asn1_create_binary(const ut8 *buffer, ut32 length) {
+/**
+ * \brief      Parse the ASN1 DER encoded buffer
+ *
+ * \param[in]  buffer  The buffer to decode
+ * \param[in]  length  The length of the buffer
+ *
+ * \return     On success returns a valid pointer, otherwise NULL
+ */
+RZ_API RZ_OWN RzASN1Object *rz_asn1_object_parse(RZ_NONNULL const ut8 *buffer, ut32 length) {
+	rz_return_val_if_fail(buffer && length > 0, NULL);
+	return asn1_create_object(buffer, length, buffer);
+}
+
+/**
+ * \brief      Allocates and initializes an RzASN1String structure
+ *
+ * \param[in]  buffer  The buffer to copy
+ * \param[in]  length  The length of the buffer
+ *
+ * \return     On success returns a valid pointer, otherwise NULL
+ */
+RZ_API RZ_OWN RzASN1Binary *rz_asn1_binary_parse(RZ_NULLABLE const ut8 *buffer, ut32 length) {
 	if (!buffer || !length) {
 		return NULL;
 	}
@@ -157,7 +181,7 @@ RZ_API RASN1Binary *rz_asn1_create_binary(const ut8 *buffer, ut32 length) {
 	if (!buf) {
 		return NULL;
 	}
-	RASN1Binary *bin = RZ_NEW0(RASN1Binary);
+	RzASN1Binary *bin = RZ_NEW0(RzASN1Binary);
 	if (!bin) {
 		free(buf);
 		return NULL;
@@ -168,19 +192,18 @@ RZ_API RASN1Binary *rz_asn1_create_binary(const ut8 *buffer, ut32 length) {
 	return bin;
 }
 
-RZ_API void rz_asn1_print_hex(RASN1Object *object, char *buffer, ut32 size, ut32 depth) {
-	ut32 i;
+static void asn1_print_hex(RzASN1Object *object, char *buffer, ut32 size, ut32 depth, bool structured) {
 	if (!object || !object->sector) {
 		return;
 	}
 	char *p = buffer;
 	char *end = buffer + size;
-	if (depth > 0 && !ASN1_STD_FORMAT) {
+	if (depth > 0 && !structured) {
 		const char *pad = rz_str_pad(' ', (depth * 2) - 2);
 		snprintf(p, end - p, "%s", pad);
 		p += strlen(pad);
 	}
-	for (i = 0; i < object->length && p < end; i++) {
+	for (ut32 i = 0; i < object->length && p < end; i++) {
 		snprintf(p, end - p, "%02x", object->sector[i]);
 		p += 2;
 	}
@@ -190,23 +213,22 @@ RZ_API void rz_asn1_print_hex(RASN1Object *object, char *buffer, ut32 size, ut32
 	}
 }
 
-#if !ASN1_STD_FORMAT
-static void rz_asn1_print_padded(RzStrBuf *sb, RASN1Object *object, int depth, const char *k, const char *v) {
+static void asn1_print_padded(RzStrBuf *sb, RzASN1Object *object, int depth, const char *k, const char *v) {
 	const char *pad = rz_str_pad(' ', (depth * 2) - 2);
 	if (object->form && !*v) {
 		return;
 	}
 	switch (object->tag) {
-	case TAG_NULL:
-	case TAG_EOC:
+	case RZ_ASN1_TAG_NULL:
+	case RZ_ASN1_TAG_EOC:
 		break;
-	case TAG_INTEGER:
-	case TAG_REAL:
+	case RZ_ASN1_TAG_INTEGER:
+	case RZ_ASN1_TAG_REAL:
 		if (*rz_str_trim_head_ro(v)) {
 			rz_strbuf_appendf(sb, "%s%s\n%s%s\n", pad, k, pad, v);
 		}
 		break;
-	case TAG_BITSTRING:
+	case RZ_ASN1_TAG_BITSTRING:
 	default:
 		if (*rz_str_trim_head_ro(v)) {
 			rz_strbuf_appendf(sb, "%s%s\n", pad, v);
@@ -214,9 +236,8 @@ static void rz_asn1_print_padded(RzStrBuf *sb, RASN1Object *object, int depth, c
 		break;
 	}
 }
-#endif
 
-static RASN1String *rz_asn1_print_hexdump_padded(RASN1Object *object, ut32 depth) {
+static RzASN1String *asn1_print_hexdump_padded(RzASN1Object *object, ut32 depth, bool structured) {
 	const char *pad;
 	ut32 i, j;
 	char readable[20] = { 0 };
@@ -224,7 +245,7 @@ static RASN1String *rz_asn1_print_hexdump_padded(RASN1Object *object, ut32 depth
 		return NULL;
 	}
 	RzStrBuf *sb = rz_strbuf_new("");
-	if (ASN1_STD_FORMAT) {
+	if (structured) {
 		pad = "                                        : ";
 	} else {
 		pad = rz_str_pad(' ', depth * 2);
@@ -248,7 +269,7 @@ static RASN1String *rz_asn1_print_hexdump_padded(RASN1Object *object, ut32 depth
 	}
 	rz_strbuf_appendf(sb, "|%-16s|", readable);
 	char *text = rz_strbuf_drain(sb);
-	RASN1String *asn1str = rz_asn1_create_string(text, true, strlen(text) + 1);
+	RzASN1String *asn1str = rz_asn1_string_parse(text, true, strlen(text) + 1);
 	if (!asn1str) {
 		/* no memory left.. */
 		free(text);
@@ -256,144 +277,147 @@ static RASN1String *rz_asn1_print_hexdump_padded(RASN1Object *object, ut32 depth
 	return asn1str;
 }
 
-RZ_API char *rz_asn1_to_string(RASN1Object *object, ut32 depth, RzStrBuf *sb) {
-	ut32 i;
-	bool root = false;
+/**
+ * \brief      Converts an the ASN1 structure to a human readable string
+ *
+ * \param      object      The ASN1 object
+ * \param[in]  depth       The padding depth
+ * \param[in]  structured  Indicates if to print its structures or not
+ * \param      sb          The RzStrBuf to write to
+ */
+RZ_API void rz_asn1_to_strbuf(RZ_NULLABLE RzASN1Object *object, ut32 depth, bool structured, RZ_NONNULL RzStrBuf *sb) {
+	rz_return_if_fail(sb);
 	if (!object) {
-		return NULL;
-	}
-	if (!sb) {
-		sb = rz_strbuf_new("");
-		root = true;
+		return;
 	}
 	// this shall not be freed. it's a pointer into the buffer.
-	RASN1String *asn1str = NULL;
+	RzASN1String *asn1str = NULL;
 	static char temp_name[4096] = { 0 };
 	const char *name = "";
 	const char *string = "";
 
 	switch (object->klass) {
-	case CLASS_UNIVERSAL: // universal
+	case RZ_ASN1_CLASS_UNIVERSAL: // universal
 		switch (object->tag) {
-		case TAG_EOC:
+		case RZ_ASN1_TAG_EOC:
 			name = "EOC";
 			break;
-		case TAG_BOOLEAN:
+		case RZ_ASN1_TAG_BOOLEAN:
 			name = "BOOLEAN";
 			if (object->sector) {
 				string = (object->sector[0] != 0) ? "true" : "false";
 			}
 			break;
-		case TAG_INTEGER:
+		case RZ_ASN1_TAG_INTEGER:
 			name = "INTEGER";
 			if (object->length < 16) {
-				rz_asn1_print_hex(object, temp_name, sizeof(temp_name), depth);
+				asn1_print_hex(object, temp_name, sizeof(temp_name), depth, structured);
 				string = temp_name;
 			} else {
-				asn1str = rz_asn1_print_hexdump_padded(object, depth);
+				asn1str = asn1_print_hexdump_padded(object, depth, structured);
 			}
 			break;
-		case TAG_BITSTRING:
+		case RZ_ASN1_TAG_BITSTRING:
 			name = "BIT_STRING";
 			if (!object->list.objects) {
 				if (object->length < 16) {
-					rz_asn1_print_hex(object, temp_name, sizeof(temp_name), depth);
+					asn1_print_hex(object, temp_name, sizeof(temp_name), depth, structured);
 					string = temp_name;
 				} else {
-					asn1str = rz_asn1_print_hexdump_padded(object, depth);
+					asn1str = asn1_print_hexdump_padded(object, depth, structured);
 				}
 			}
 			break;
-		case TAG_OCTETSTRING:
+		case RZ_ASN1_TAG_OCTETSTRING:
 			name = "OCTET_STRING";
 			if (rz_str_is_printable_limited((const char *)object->sector, object->length)) {
 				asn1str = rz_asn1_stringify_string(object->sector, object->length);
 			} else if (!object->list.objects) {
 				if (object->length < 16) {
-					rz_asn1_print_hex(object, temp_name, sizeof(temp_name), depth);
+					asn1_print_hex(object, temp_name, sizeof(temp_name), depth, structured);
 					string = temp_name;
 				} else {
-					asn1str = rz_asn1_print_hexdump_padded(object, depth);
+					asn1str = asn1_print_hexdump_padded(object, depth, structured);
 				}
 			}
 			break;
-		case TAG_NULL:
+		case RZ_ASN1_TAG_NULL:
 			name = "NULL";
 			break;
-		case TAG_OID:
+		case RZ_ASN1_TAG_OID:
 			name = "OBJECT_IDENTIFIER";
 			asn1str = rz_asn1_stringify_oid(object->sector, object->length);
 			break;
-		case TAG_OBJDESCRIPTOR:
+		case RZ_ASN1_TAG_OBJDESCRIPTOR:
 			name = "OBJECT_DESCRIPTOR";
 			break;
-		case TAG_EXTERNAL:
+		case RZ_ASN1_TAG_EXTERNAL:
 			name = "EXTERNAL";
 			break;
-		case TAG_REAL:
+		case RZ_ASN1_TAG_REAL:
 			name = "REAL";
-			asn1str = rz_asn1_print_hexdump_padded(object, depth);
+			asn1str = asn1_print_hexdump_padded(object, depth, structured);
 			break;
-		case TAG_ENUMERATED:
+		case RZ_ASN1_TAG_ENUMERATED:
 			name = "ENUMERATED";
 			break;
-		case TAG_EMBEDDED_PDV:
+		case RZ_ASN1_TAG_EMBEDDED_PDV:
 			name = "EMBEDDED_PDV";
 			break;
-		case TAG_UTF8STRING:
+		case RZ_ASN1_TAG_UTF8STRING:
 			name = "UTF8String";
 			asn1str = rz_asn1_stringify_string(object->sector, object->length);
 			break;
-		case TAG_SEQUENCE:
+		case RZ_ASN1_TAG_SEQUENCE:
 			name = "SEQUENCE";
 			break;
-		case TAG_SET:
+		case RZ_ASN1_TAG_SET:
 			name = "SET";
 			break;
-		case TAG_NUMERICSTRING:
+		case RZ_ASN1_TAG_NUMERICSTRING:
 			name = "NumericString";
 			asn1str = rz_asn1_stringify_string(object->sector, object->length);
 			break;
-		case TAG_PRINTABLESTRING:
+		case RZ_ASN1_TAG_PRINTABLESTRING:
 			name = "PrintableString"; // ASCII subset
 			asn1str = rz_asn1_stringify_string(object->sector, object->length);
 			break;
-		case TAG_T61STRING:
+		case RZ_ASN1_TAG_T61STRING:
 			name = "TeletexString"; // aka T61String
 			asn1str = rz_asn1_stringify_string(object->sector, object->length);
 			break;
-		case TAG_VIDEOTEXSTRING:
+		case RZ_ASN1_TAG_VIDEOTEXSTRING:
 			name = "VideotexString";
 			asn1str = rz_asn1_stringify_string(object->sector, object->length);
 			break;
-		case TAG_IA5STRING:
+		case RZ_ASN1_TAG_IA5STRING:
 			name = "IA5String"; // ASCII
 			asn1str = rz_asn1_stringify_string(object->sector, object->length);
 			break;
-		case TAG_UTCTIME:
+		case RZ_ASN1_TAG_UTCTIME:
 			name = "UTCTime";
 			asn1str = rz_asn1_stringify_utctime(object->sector, object->length);
 			break;
-		case TAG_GENERALIZEDTIME:
+		case RZ_ASN1_TAG_GENERALIZEDTIME:
 			name = "GeneralizedTime";
 			asn1str = rz_asn1_stringify_time(object->sector, object->length);
 			break;
-		case TAG_GRAPHICSTRING:
+		case RZ_ASN1_TAG_GRAPHICSTRING:
 			name = "GraphicString";
 			asn1str = rz_asn1_stringify_string(object->sector, object->length);
 			break;
-		case TAG_VISIBLESTRING:
+		case RZ_ASN1_TAG_VISIBLESTRING:
 			name = "VisibleString"; // ASCII subset
 			asn1str = rz_asn1_stringify_string(object->sector, object->length);
 			break;
-		case TAG_GENERALSTRING:
+		case RZ_ASN1_TAG_GENERALSTRING:
 			name = "GeneralString";
 			break;
-		case TAG_UNIVERSALSTRING:
+		case RZ_ASN1_TAG_UNIVERSALSTRING:
 			name = "UniversalString";
 			asn1str = rz_asn1_stringify_string(object->sector, object->length);
 			break;
-		case TAG_BMPSTRING:
+		case RZ_ASN1_TAG_BMPSTRING:
 			name = "BMPString";
 			asn1str = rz_asn1_stringify_string(object->sector, object->length);
 			break;
@@ -403,15 +427,15 @@ RZ_API char *rz_asn1_to_string(RASN1Object *object, ut32 depth, RzStrBuf *sb) {
 			break;
 		}
 		break;
-	case CLASS_APPLICATION:
+	case RZ_ASN1_CLASS_APPLICATION:
 		snprintf(temp_name, sizeof(temp_name), "Application_%u", object->tag);
 		name = temp_name;
 		break;
-	case CLASS_CONTEXT:
+	case RZ_ASN1_CLASS_CONTEXT:
 		snprintf(temp_name, sizeof(temp_name), "Context [%u]", object->tag); // Context
 		name = temp_name;
 		break;
-	case CLASS_PRIVATE:
+	case RZ_ASN1_CLASS_PRIVATE:
 		snprintf(temp_name, sizeof(temp_name), "Private_%u", object->tag);
 		name = temp_name;
 		break;
@@ -419,39 +443,63 @@ RZ_API char *rz_asn1_to_string(RASN1Object *object, ut32 depth, RzStrBuf *sb) {
 	if (asn1str) {
 		string = asn1str->string;
 	}
-	if (ASN1_STD_FORMAT) {
+	if (structured) {
 		rz_strbuf_appendf(sb, "%4" PFMT64d "  ", object->offset);
 		rz_strbuf_appendf(sb, "%4u:%2d: %s %-20s: %s\n", object->length,
 			depth, object->form ? "cons" : "prim", name, string);
-		rz_asn1_free_string(asn1str);
+		rz_asn1_string_free(asn1str);
 		if (object->list.objects) {
-			for (i = 0; i < object->list.length; i++) {
-				rz_asn1_to_string(object->list.objects[i], depth + 1, sb);
+			for (ut32 i = 0; i < object->list.length; i++) {
+				rz_asn1_to_strbuf(object->list.objects[i], depth + 1, structured, sb);
 			}
 		}
 	} else {
-		rz_asn1_print_padded(sb, object, depth, name, string);
-		rz_asn1_free_string(asn1str);
+		asn1_print_padded(sb, object, depth, name, string);
+		rz_asn1_string_free(asn1str);
 		if (object->list.objects) {
-			for (i = 0; i < object->list.length; i++) {
-				RASN1Object *obj = object->list.objects[i];
-				rz_asn1_to_string(obj, depth + 1, sb);
+			for (ut32 i = 0; i < object->list.length; i++) {
+				RzASN1Object *obj = object->list.objects[i];
+				rz_asn1_to_strbuf(obj, depth + 1, structured, sb);
 			}
 		}
 	}
-	return root ? rz_strbuf_drain(sb) : NULL;
 }
 
-RZ_API void rz_asn1_free_object(RASN1Object *object) {
-	ut32 i;
+/**
+ * \brief      Converts an the ASN1 structure to a human readable string
+ *
+ * \param      object      The ASN1 object
+ * \param[in]  depth       The padding depth
+ * \param[in]  structured  Indicates if to print its structures or not
+ *
+ * \return     On success returns a valid pointer, otherwise NULL
+ */
+RZ_API RZ_OWN char *rz_asn1_to_string(RZ_NULLABLE RzASN1Object *object, ut32 depth, bool structured) {
+	if (!object) {
+		return NULL;
+	}
+	RzStrBuf *sb = rz_strbuf_new("");
+	if (!sb) {
+		return NULL;
+	}
+	rz_asn1_to_strbuf(object, depth, structured, sb);
+	return rz_strbuf_drain(sb);
+}
+
+/**
+ * \brief      Frees an RzASN1Object structure
+ *
+ * \param      object  The ASN1 object to be freed
+ */
+RZ_API void rz_asn1_object_free(RZ_NULLABLE RzASN1Object *object) {
 	if (!object) {
 		return;
 	}
 	// This shall not be freed. it's a pointer into the buffer.
 	object->sector = NULL;
 	if (object->list.objects) {
-		for (i = 0; i < object->list.length; i++) {
-			rz_asn1_free_object(object->list.objects[i]);
+		for (ut32 i = 0; i < object->list.length; i++) {
+			rz_asn1_object_free(object->list.objects[i]);
 		}
 		RZ_FREE(object->list.objects);
 	}
@@ -460,9 +508,15 @@ RZ_API void rz_asn1_free_object(RASN1Object *object) {
 	free(object);
 }
 
-RZ_API void rz_asn1_free_binary(RASN1Binary *bin) {
-	if (bin) {
-		free(bin->binary);
-		free(bin);
+/**
+ * \brief      Frees an RzASN1Binary structure
+ *
+ * \param      bin  The ASN1 binary to be freed
+ */
+RZ_API void rz_asn1_binary_free(RZ_NULLABLE RzASN1Binary *bin) {
+	if (!bin) {
+		return;
 	}
+	free(bin->binary);
+	free(bin);
 }
