@@ -15,7 +15,7 @@ typedef struct RZARFP {
 /* Offset passed is always the real io->off of the inspected file,
  * the functions automatically translate it to relative offset within the archive */
 RZ_API RzArFp *ar_open_file(const char *arname, int perm, const char *filename);
-RZ_API RzList *ar_open_all(const char *arname, int perm);
+RZ_API RzList /*<RzArFp *>*/ *ar_open_all(const char *arname, int perm);
 RZ_API void ar_close(RzArFp *f);
 RZ_API int ar_read_at(RzArFp *f, ut64 off, void *buf, int count);
 RZ_API int ar_write_at(RzArFp *f, ut64 off, void *buf, int count);
